from datetime import date
from icemac.ab.calendar.browser.interfaces import IEventDescription
from icemac.ab.calendar.browser.interfaces import UnknownLanguageError
from icemac.ab.calendar.browser.renderer.interfaces import IRenderer
from icemac.ab.calendar.browser.renderer.table import Table, TableEvent
from icemac.ab.calendar.interfaces import ICalendarDisplaySettings, IEvent
from icemac.addressbook.interfaces import IEntity
from zope.testbrowser.browser import LinkNotFoundError
from mock import Mock, call
from zope.interface.verify import verifyObject
import lxml
import pytest
import zope.component


def test_table__Table__1():
    """Table_fulfills_IRenderer_interface."""
    assert verifyObject(IRenderer, Table(None, None, None))


def test_table__Table__render__1(
        address_book, browser, EventFactory, CategoryFactory, DateTime):
    """It renders two events at the same time with two `dt` elements."""
    today = date.today()
    EventFactory(address_book, alternative_title=u'event1',
                 datetime=DateTime(today.year, today.month, 22, 16, 14),
                 category=CategoryFactory(address_book, u'foo'))
    EventFactory(address_book, alternative_title=u'event2',
                 datetime=DateTime(today.year, today.month, 22, 16, 14),
                 category=CategoryFactory(address_book, u'bar'))
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    assert 2 == len(browser.etree.xpath('//dt'))


def test_table__Table__render__2(address_book, browser):
    """It translates weekdays into the language of the user."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    assert 'Sonntag' not in browser.contents  # default English locale
    browser.lang('de-DE')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    assert 'Sonntag' in browser.contents  # switched to German locale


def test_table__Table__render__3(
        address_book, browser, CategoryFactory, DateTime):
    """It renders the day numbers as links to add a new event."""
    CategoryFactory(address_book, u'example')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    assert browser.getLink('15').url.startswith(browser.EVENT_ADD_URL)
    browser.getLink('15').click()
    assert (DateTime.format(date.today().replace(day=15)) ==
            browser.getControl('date').value)
    assert '12:00' == browser.getControl('time').value
    # Whole day event:
    assert browser.getControl('yes').selected
    browser.getControl('Add', index=1).click()
    assert '"example" added.' == browser.message


def test_table__Table__render__4(address_book, browser):
    """It renders day numbers are text for visitors."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink('15')


@pytest.fixture('function')
def TableEventUFactory(RequestFactory):
    """Create a `TableEvent` view set up with some mocks for unit testing."""
    def create_table_event(text, lang):
        view = TableEvent()
        view.context = Mock()
        view.context.getText.side_effect = [
            UnknownLanguageError, UnknownLanguageError, text]
        view.request = RequestFactory(HTTP_ACCEPT_LANGUAGE=lang)
        return view
    return create_table_event


def test_table__TableEvent__text__1(TableEventUFactory):
    """It tries to find a language code `getText` understands."""
    view = TableEventUFactory('Foo', 'de_DE')
    assert 'Foo' == view.text()
    assert ([call(u'de_DE'), call(u'de'), call()] ==
            view.context.getText.call_args_list)


def test_table__TableEvent__text__2(TableEventUFactory):
    """It returns "Edit" if the text is empty.

    So the user can edit events with empty text.
    """
    view = TableEventUFactory('', lang='de_DE')
    assert u'Edit' == view.text()


@pytest.fixture('function')
def TableEventIFactory(address_book, utc_time_zone_pref, RequestFactory):
    """Get a `TableEvent` view for integration testing."""
    def get_table_event(event, field_names=[], request_kw={}):
        request = RequestFactory(**request_kw)
        event_entity = IEntity(IEvent)
        ICalendarDisplaySettings(
            address_book.calendar).event_additional_fields = [
                event_entity.getRawField(x) for x in field_names]
        event_description = IEventDescription(event)
        return zope.component.getMultiAdapter(
            (event_description, request), name='table-event')
    return get_table_event


def test_table__TableEvent__render__1(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders nothing if no event additional field is selected."""
    event = EventFactory(address_book, datetime=DateTime.now)
    assert 'class="info"' not in TableEventIFactory(event, [])()


def test_table__TableEvent__render__2(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders a single selected event additional field not as a list."""
    event = EventFactory(address_book, datetime=DateTime.now,
                         external_persons=[u'Foo', u'Bar'])
    assert ('<span class="info">Bar, Foo</span>' in
            TableEventIFactory(event, ['persons'])())


def test_table__TableEvent__render__3(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders multiple selected event additional fields as a list."""
    event = EventFactory(address_book,
                         datetime=DateTime.now,
                         external_persons=[u'Foo', u'Bar'], text=u'Cool!')
    etree = lxml.etree.HTML(
        TableEventIFactory(event, ['persons', 'text'])())
    assert [
        'Bar, Foo',
        'Cool!'
    ] == etree.xpath('//ul[@class="info"]/li/text()')


def test_table__TableEvent__info_1(
        address_book, EventFactory, FieldFactory, DateTime,
        TableEventIFactory):
    """It renders user defined fields."""
    field_name = FieldFactory(
        address_book, IEvent, u'Int', u'reservations').__name__
    event = EventFactory(
        address_book,
        **{'datetime': DateTime.now, 'text': u'Text1', field_name: 42})
    assert ([{u'info': u'Text1'}, {u'info': u'42'}] ==
            TableEventIFactory(event, ['text', field_name]).info())


def test_table__TableEvent__time__1(
        address_book, EventFactory, DateTime, TableEventIFactory,
        TimeZonePrefFactory):
    """It converts the time to the time zone of the user."""
    event = EventFactory(
        address_book, datetime=DateTime(2013, 11, 2, 9, 27))
    TimeZonePrefFactory('Australia/Currie')
    assert u'20:27' == TableEventIFactory(event).time()


def test_table__TableEvent__time__2(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders "Uhr" if the requested language is German."""
    event = EventFactory(
        address_book, datetime=DateTime(2013, 11, 2, 9, 47))
    view = TableEventIFactory(event, request_kw={'HTTP_ACCEPT_LANGUAGE': 'de'})
    assert u'09:47 Uhr' == view.time()


def test_table__TableEvent__time__3(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders "AM" if the requested language is English."""
    event = EventFactory(
        address_book, datetime=DateTime(2013, 11, 2, 9, 47))
    view = TableEventIFactory(event, request_kw={'HTTP_ACCEPT_LANGUAGE': 'en'})
    assert u'9:47 AM' == view.time()


def test_table__TableEvent__time__4(
        address_book, EventFactory, DateTime, TableEventIFactory):
    """It renders a zero width space for a whole day event."""
    event = EventFactory(
        address_book, datetime=DateTime(2013, 11, 2, 18),
        whole_day_event=True)
    assert u'&#x200b;' == TableEventIFactory(event).time()



from zope.security.interfaces import Unauthorized
from zope.testbrowser.browser import LinkNotFoundError
import pytest

EVENT_VIEW_CONFIGURATION_ADD_TEXT = 'event view configuration'


def test_masterdata__Table__1(address_book, browser):
    """It allows to navigate to the event views list."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MASTERDATA_URL)
    browser.getLink('Event views').click()
    assert browser.url == browser.CALENDAR_MASTERDATA_EVENTVIEW_URL


def test_masterdata__Table__2(address_book, browser):
    """It renders a message if there are no event view configurations yet."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MASTERDATA_EVENTVIEW_URL)
    assert 'No event views defined yet.' in browser.contents


@pytest.mark.parametrize('login', ('cal-visitor', 'cal-editor'))
def test_masterdata__Table__3(address_book, browser, login):
    """It renders no add link for any calendar user."""
    browser.login(login)
    browser.open(browser.CALENDAR_MASTERDATA_EVENTVIEW_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink(EVENT_VIEW_CONFIGURATION_ADD_TEXT)


def test_masterdata__Table__4(address_book, browser):
    """It prevents access for anonymous."""
    browser.handleErrors = False  # needed to catch exception
    with pytest.raises(Unauthorized):
        browser.open(browser.CALENDAR_MASTERDATA_EVENTVIEW_URL)


def test_masterdata__Add__1(address_book, browser):
    """It allows administrators to add a new category in the list."""
    browser.login('mgr')
    browser.open(browser.CALENDAR_MASTERDATA_EVENTVIEW_URL)
    browser.getLink(EVENT_VIEW_CONFIGURATION_ADD_TEXT).click()
    assert browser.CALENDAR_EVENTVIEW_CONFIGURATION_ADD_URL == browser.url
    browser.getControl('title').value = 'default'
    browser.getControl('Add').click()
    assert '"default" added.' == browser.message
    # The new configuration shows up in the list:
    assert '>default<' in browser.contents


def test_masterdata__Add__2(
        address_book, EventViewConfigurationFactory, browser):
    """It prevents adding a new config with an already existing title."""
    EventViewConfigurationFactory(address_book, u'default')
    browser.login('mgr')
    browser.open(browser.CALENDAR_EVENTVIEW_CONFIGURATION_ADD_URL)
    browser.getControl('title').value = 'default'
    browser.getControl('Add').click()
    assert 'There were some errors.' in browser.contents
    assert 'This title is already used for an ' in browser.contents


@pytest.mark.parametrize('login', ('cal-visitor', 'cal-editor'))
def test_masterdata__Add__3(address_book, browser, login):
    """It is not accessible for any calendar user."""
    browser.login(login)
    browser.assert_forbidden(browser.CALENDAR_EVENTVIEW_CONFIGURATION_ADD_URL)


def test_masterdata__Edit__1(
        address_book, EventViewConfigurationFactory, CategoryFactory, browser):
    """It allows to edit a category."""
    EventViewConfigurationFactory(address_book, u'default')
    CategoryFactory(address_book, u'foo')
    CategoryFactory(address_book, u'bar')
    browser.login('mgr')
    browser.open(browser.CALENDAR_MASTERDATA_EVENTVIEW_URL)
    browser.getLink('default').click()
    assert browser.CALENDAR_EVENTVIEW_CONFIGURATION_EDIT_URL == browser.url
    assert 'default' == browser.getControl('title').value
    browser.getControl('title').value = 'alternative'
    browser.getControl('start date').displayValue = ['3 days in past']
    browser.getControl('duration').displayValue = ['3 weeks']
    browser.getControl('categories').displayValue = ['bar']
    browser.getControl('show fields').displayValue = ['persons']
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    # The changed category name shows up in the list:
    assert 'alternative' in browser.contents
    browser.getLink('alternative').click()
    assert browser.getControl('title').value == 'alternative'
    assert browser.getControl('start date').displayValue == ['3 days in past']
    assert browser.getControl('duration').displayValue == ['3 weeks']
    assert browser.getControl('categories').displayValue == ['bar']
    assert browser.getControl('show fields').displayValue == ['persons']


def test_masterdata__Edit__2(
        address_book, EventViewConfigurationFactory, browser):
    """It prevents changing a category title to an existing one."""
    EventViewConfigurationFactory(address_book, u'default')
    EventViewConfigurationFactory(address_book, u'alternative')
    browser.login('mgr')
    browser.open(browser.CALENDAR_EVENTVIEW_CONFIGURATION_EDIT_URL)
    browser.getControl('title').value = 'alternative'
    browser.getControl('Apply').click()
    assert 'There were some errors.' in browser.contents
    assert 'This title is already used for an ' in browser.contents


@pytest.mark.parametrize('login', ('cal-visitor', 'cal-editor'))
def test_masterdata__Edit__3(
        address_book, EventViewConfigurationFactory, browser, login):
    """It allows calendar users only to see the event view configuration data.

    But they cannot change or delete them.
    """
    EventViewConfigurationFactory(address_book, u'foo')
    browser.login(login)
    browser.open(browser.CALENDAR_EVENTVIEW_CONFIGURATION_EDIT_URL)
    # There are no fields and no delete button:
    assert (['form.buttons.apply', 'form.buttons.cancel'] ==
            browser.all_control_names)


def test_masterdata__Delete__1(
        address_book, EventViewConfigurationFactory, browser):
    """It allows to delete an event view configuration."""
    EventViewConfigurationFactory(address_book, u'default')
    browser.login('mgr')
    browser.open(browser.CALENDAR_EVENTVIEW_CONFIGURATION_EDIT_URL)
    browser.getControl('Delete').click()
    assert browser.CALENDAR_EVENTVIEW_CONFIGURATION_DELETE_URL == browser.url
    assert ('Do you really want to delete this event view configuration?' in
            browser.contents)
    browser.getControl('Yes').click()
    assert '"default" deleted.' == browser.message


@pytest.mark.parametrize('login', ('cal-visitor', 'cal-editor'))
def test_masterdata__Delete__2(
        address_book, EventViewConfigurationFactory, browser, login):
    """It is not accessible for any calendar user."""
    EventViewConfigurationFactory(address_book, u'foo')
    browser.login(login)
    browser.assert_forbidden(
        browser.CALENDAR_EVENTVIEW_CONFIGURATION_DELETE_URL)



from datetime import date, datetime, time
from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.browser.base
import icemac.ab.calendar.browser.interfaces
import icemac.ab.calendar.event
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.browser.metadata
import icemac.addressbook.interfaces
import icemac.addressbook.preferences.utils
import pytz
import z3c.form.button
import z3c.form.error
import z3c.form.form
import z3c.form.interfaces
import z3c.form.object
import z3c.form.term
import zope.component
import zope.traversing.browser


MIDNIGHT = time(0, 0)
NOON = time(12, 0)
EVENT_CONFIRMATION_FIELDS = (
    'category', 'alternative_title', 'whole_day_event', 'datetime')


def date_from_iso_string(string):
    """Convert an YYYY-MM-DD string to a python date object.

    Returns `None` if the string is `None`.

    """
    if string is None:
        return None
    return date(*tuple(int(x) for x in string.split('-')))


@zope.component.adapter(icemac.ab.calendar.interfaces.IEvent)
@zope.interface.implementer(
    icemac.ab.calendar.browser.interfaces.IEventDatetime)
class EventDatetime(object):
    """Adapter to edit an event using the IDatetime object field.

    Needs to be registered via ZCML, as grok has no model based security.
    """

    def __init__(self, context):
        self.context = self.__parent__ = context

    @property
    def datetime(self):
        return icemac.ab.calendar.browser.interfaces.IDatetime(self.context)

    @datetime.setter
    def datetime(self, value):
        self.context.datetime = value.datetime
        self.context.whole_day_event = value.whole_day_event


class Datetime(grok.Adapter):
    """Adapter storing the IDatetime field data and computing `datetime`."""

    grok.context(icemac.ab.calendar.interfaces.IEvent)
    grok.implements(icemac.ab.calendar.browser.interfaces.IDatetime)

    date = None
    time = None
    whole_day_event = None
    _datetime = None

    def __init__(self, context=None, whole_day_event=None):
        super(Datetime, self).__init__(context)
        self.whole_day_event = whole_day_event
        if isinstance(self.context, datetime):
            self._datetime = self.context
        elif self.context is not None:
            # context seems to be an IEvent
            self.__parent__ = context
            self._datetime = self.context.datetime
            self.whole_day_event = self.context.whole_day_event
        if self._datetime:
            local_datetime = self._timezone.normalize(self._datetime)
            self.date = local_datetime.date()
            self.time = local_datetime.time()

    @property
    def datetime(self):
        if self.whole_day_event:
            time = NOON
        else:
            time = self.time
        return pytz.utc.normalize(self._timezone.localize(
            datetime.combine(self.date, time)))

    @property
    def _timezone(self):
        return icemac.addressbook.preferences.utils.get_time_zone()


# Factory needed for the add form to initially store the IDatetime values:
z3c.form.object.registerFactoryAdapter(
    icemac.ab.calendar.browser.interfaces.IDatetime, Datetime)


class SourceFactoryMissingCollectionTermsSource(
        z3c.form.term.MissingCollectionTermsSource):
    """MissingCollectionTermsSource adapted to zc.sourcefactory.

    We need a zc.sourcefactory as base creating the `ITerms` adapter to
    be able to `_makeMissingTerm()` properly.
    """

    def _makeMissingTerm(self, value):
        term = self.terms.getTerm(value)
        term.title = _(u'Missing: ${value}', mapping=dict(value=term.title))
        return term


NoLongerAllowedPerson = z3c.form.error.ErrorViewMessage(
    _('Please deselect the persons prefixed with "Missing:" They are no '
      'longer in the list of available persons. (Maybe reconsider the '
      'following setting: Master Data > Calendar > Settings > '
      'Person keyword)'),
    error=zope.schema.interfaces.WrongContainedType,
    field=icemac.ab.calendar.interfaces.IBaseEvent['persons'])


class EventFields(object):
    """Form fields to add or edit an event."""

    interface = None

    @property
    def fields(self):
        fields = icemac.addressbook.interfaces.IEntity(self.interface)
        field_values = []
        for name, field in fields.getFields():
            if name == 'whole_day_event':
                continue
            elif name == 'datetime':
                field_values.append(
                    icemac.ab.calendar.browser.interfaces.IEventDatetime[
                        'datetime'])
            else:
                field_values.append(field)
        return z3c.form.field.Fields(*field_values)


class Add(EventFields, icemac.addressbook.browser.base.BaseAddForm):
    """Add form for an event."""

    title = _(u'Add new event')
    interface = icemac.ab.calendar.interfaces.IEvent
    class_ = icemac.ab.calendar.event.Event
    next_url = 'parent'
    ignoreContext = False

    def getContent(self):
        date = date_from_iso_string(self.request.get('date'))
        if date is not None:
            timezone = icemac.addressbook.preferences.utils.get_time_zone()
            selected_datetime = timezone.localize(datetime.combine(date, NOON))
            whole_day_event = True
        else:
            selected_datetime = None
            whole_day_event = False
        data = {'datetime': Datetime(selected_datetime, whole_day_event)}
        return data


class Edit(EventFields, icemac.addressbook.browser.base.GroupEditForm):
    """Edit for for an event."""

    title = _('Edit event')
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    interface = icemac.ab.calendar.interfaces.IEvent
    next_url = 'parent'
    z3c.form.form.extends(icemac.addressbook.browser.base.GroupEditForm,
                          ignoreFields=True)

    @z3c.form.button.buttonAndHandler(
        _('Clone event'), name='clone_event',
        condition=icemac.addressbook.browser.base.can_access('@@clone.html'))
    def handleCloneEvent(self, action):
        self.redirect_to_next_url('object', 'clone.html')

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete',
        condition=icemac.addressbook.browser.base.can_access('@@delete.html'))
    def handleDelete(self, action):
        self.redirect_to_next_url('object', 'delete.html')


class Delete(icemac.addressbook.browser.base.BaseDeleteForm):
    """Confirmation when deleting an event."""

    title = _('Delete event')
    label = _(u'Do you really want to delete this event?')
    interface = icemac.ab.calendar.interfaces.IEvent
    field_names = EVENT_CONFIRMATION_FIELDS


class Clone(icemac.addressbook.browser.base.BaseCloneForm):
    """Clone event with confirmation."""

    title = _('Clone event')
    label = _(u'Do you really want to clone this event?')
    interface = icemac.ab.calendar.interfaces.IEvent
    field_names = EVENT_CONFIRMATION_FIELDS


class CustomizeRecurredEvent(icemac.ab.calendar.browser.base.View):
    """Prepare customization of a RecurredEvent."""

    def __call__(self):
        self.session['recurring-event-name'] = self.request.form['event']
        self.session['recurred-event-date'] = date_from_iso_string(
            self.request.form['date'])
        if icemac.addressbook.browser.base.can_access_uri_part(
                self.context, self.request, 'addFromRecurredEvent.html'):
            target_view = 'addFromRecurredEvent.html'
        else:
            target_view = 'viewRecurredEvent.html'
        self.request.response.redirect(self.url(self.context, target_view))
        return ''


class RecurredEventFormMixIn(object):
    """Mix-In providing data of recurred event from session."""

    def getContent(self):
        recurring_events = zope.component.getUtility(
            icemac.ab.calendar.interfaces.IRecurringEvents)
        recurring_event = recurring_events[
            self.session['recurring-event-name']]
        date = self.session['recurred-event-date']
        timezone = icemac.addressbook.preferences.utils.get_time_zone()
        datetime = timezone.normalize(recurring_event.datetime)
        datetime = datetime.replace(
            year=date.year, month=date.month, day=date.day, tzinfo=None)
        # Fix possible DST offset:
        datetime = timezone.localize(datetime)
        data = icemac.ab.calendar.event.get_event_data_from_recurring_event(
            recurring_event, datetime)
        data['datetime'] = Datetime(
            data['datetime'], data.pop('whole_day_event'))
        return data


class AddFromRecurredEvent(icemac.ab.calendar.browser.base.View,
                           RecurredEventFormMixIn,
                           EventFields,
                           icemac.addressbook.browser.base.BaseAddForm):
    """Add form for changing a recurred event."""

    title = _(u'Edit recurred event')
    interface = icemac.ab.calendar.interfaces.IEvent
    class_ = icemac.ab.calendar.event.Event
    next_url = 'parent'
    ignoreContext = False
    buttons = z3c.form.button.Buttons()
    handlers = z3c.form.button.Handlers()

    # Rename `Add` button to `Apply`.
    @z3c.form.button.buttonAndHandler(_('Apply'), name='add')
    def handleAdd(self, action):
        super(AddFromRecurredEvent, self).handleAdd(self, action)

    @z3c.form.button.buttonAndHandler(_('Delete'), name='delete')
    def handleDelete(self, action):
        self.request.response.redirect(
            self.url(self.context, 'delete-recurred-event.html'))

    # Copy over the cancel button and handler:
    buttons += icemac.addressbook.browser.base.BaseAddForm.buttons.select(
        'cancel')
    handlers += icemac.addressbook.browser.base.BaseAddForm.handlers.copy()


class ViewRecurredEvent(icemac.ab.calendar.browser.base.View,
                        RecurredEventFormMixIn,
                        EventFields,
                        icemac.addressbook.browser.base.BaseEditForm):
    """View form for a recurred event."""

    label = _(u'View recurred event')
    interface = icemac.ab.calendar.interfaces.IEvent
    mode = z3c.form.interfaces.DISPLAY_MODE
    next_url = 'parent'


class DeleteRecurredEvent(icemac.ab.calendar.browser.base.View,
                          RecurredEventFormMixIn,
                          EventFields,
                          icemac.addressbook.browser.base.BaseDeleteForm):
    """Add form for deleting a recurred event."""

    title = _('Delete recurred event')
    label = _(u'Do you really want to delete this recurred event?')
    interface = icemac.ab.calendar.interfaces.IEvent
    field_names = EVENT_CONFIRMATION_FIELDS
    next_url_after_delete = 'object'

    @property
    def status_title(self):
        content = self.getContent()
        if content['alternative_title']:
            return content['alternative_title']
        return icemac.addressbook.interfaces.ITitle(content['category'])

    def _do_delete(self):
        content = self.getContent()
        icemac.addressbook.utils.create_and_add(
            self.context, icemac.ab.calendar.event.Event,
            category=content['category'],
            datetime=content['datetime'].datetime,
            whole_day_event=content['datetime'].whole_day_event,
            deleted=True)


class RecurredEventAbsoluteURL(zope.traversing.browser.AbsoluteURL,
                               icemac.ab.calendar.browser.base.View):
    """URL to customize a recurred event."""

    def __str__(self):
        """URL of the recurrend event."""
        return self.url(self.context.__parent__, 'customize-recurred-event',
                        event=self.context.recurring_event.__name__,
                        date=self.context.datetime.date().isoformat())

    __call__ = __str__



from .interfaces import DATE_INDEX
import icemac.ab.calendar.calendar
import icemac.ab.calendar.category
import icemac.ab.calendar.eventview.interfaces
import icemac.ab.calendar.eventview.model
import icemac.ab.calendar.interfaces
import icemac.addressbook.addressbook
import zc.catalog.catalogindex
import zope.catalog.interfaces
import zope.component
import zope.component.hooks


@zope.component.adapter(
    icemac.addressbook.addressbook.AddressBookCreated)
def install_calendar(event):
    """Install the calendar in the newly created addressbook."""
    address_book = event.address_book
    with zope.component.hooks.site(address_book):
        icemac.addressbook.addressbook.create_and_register(
            address_book, 'calendar', icemac.ab.calendar.calendar.Calendar,
            icemac.ab.calendar.interfaces.ICalendar)
        icemac.addressbook.addressbook.create_and_register(
            address_book, 'calendar_categories',
            icemac.ab.calendar.category.CategoryContainer,
            icemac.ab.calendar.interfaces.ICategories)
        update_calendar_infrastructure(address_book)


def update_calendar_infrastructure(address_book):
    """Update the calendar infrastructure to install new components."""
    with zope.component.hooks.site(address_book):
        catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
        if DATE_INDEX not in catalog:
            catalog[DATE_INDEX] = zc.catalog.catalogindex.DateTimeValueIndex(
                'datetime', icemac.ab.calendar.interfaces.IEventDateTime,
                resolution=1)
            catalog.updateIndex(catalog.get(DATE_INDEX))
        icemac.addressbook.addressbook.add_entity_to_order(
            address_book.orders, icemac.ab.calendar.interfaces.IEvent)
        icemac.addressbook.addressbook.create_and_register(
            address_book, 'calendar_recurring_events',
            icemac.ab.calendar.event.RecurringEventContainer,
            icemac.ab.calendar.interfaces.IRecurringEvents)
        icemac.addressbook.addressbook.add_entity_to_order(
            address_book.orders, icemac.ab.calendar.interfaces.IRecurringEvent)

        icemac.addressbook.addressbook.create_and_register(
            address_book, 'calendar_eventviews',
            icemac.ab.calendar.eventview.model.EventViewContainer,
            icemac.ab.calendar.eventview.interfaces.IEventViewContainer)



# -*- coding: utf-8 -*-
from icemac.ab.calendar.browser.renderer.base import Calendar
from icemac.ab.calendar.browser.renderer.interfaces import IRenderer
from zope.interface.verify import verifyObject
import pytest


def test_base__Calendar__1():
    """It fulfills the `IRenderer` interface."""
    assert verifyObject(IRenderer, Calendar(None, None, None))


def test_base__Calendar__write__1():
    """It does not fail on non-ASCII chars."""
    calendar = Calendar(None, None, None)
    calendar.write(u'aäöü%s', u'ßen')
    assert u'aäöüßen\n' == calendar.read()


def test_base__Calendar__update__1():
    """It does nothing.

    Test is only here to complete test coverage.
    """
    calendar = Calendar(None, None, None)
    calendar.update()


def test_base__Calendar__render__1():
    """It raises an exception: method has to be implemented by child classes.

    Test is only here to complete test coverage.
    """
    calendar = Calendar(None, None, None)
    with pytest.raises(NotImplementedError):
        calendar.render()



import icemac.ab.calendar.install
import icemac.addressbook.addressbook
import icemac.addressbook.generations.utils


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(address_book):
    """Install the calendar into each existing address book."""
    icemac.ab.calendar.install.install_calendar(
        icemac.addressbook.addressbook.AddressBookCreated(address_book))



from __future__ import unicode_literals

from .browser import EventView
from icemac.ab.calendar.interfaces import IEvent
from icemac.addressbook.interfaces import IEntity
import mock
import zope.publisher.testing


def test_browser__EventView__1(
        address_book, EventViewConfigurationFactory, EventFactory,
        CategoryFactory, RecurringEventFactory, DateTime, browser):
    """It renders a listing of event views."""
    EventFactory(
        address_book, category=CategoryFactory(address_book, 'Party'),
        datetime=DateTime.now)
    RecurringEventFactory(
        address_book, category=CategoryFactory(address_book, 'Lesson'),
        datetime=DateTime.add(DateTime.now, days=3),
        period='weekly')
    EventViewConfigurationFactory(
        address_book, '-1 day for 1 week', start=-1, duration=7)

    browser.login('cal-visitor')
    browser.lang(b'en')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    browser.getLink('Event views').click()
    assert browser.CALENDAR_EVENT_VIEWS_URL == browser.url
    assert '>Party<' in browser.ucontents
    assert '>Lesson<' in browser.ucontents
    assert 'Please create an "Event view"' not in browser.ucontents


def test_browser__EventView__2(address_book, browser):
    """It renders a hint if there is no event view defined in master data."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
    assert 'Please create an "Event view"' in browser.ucontents


def test_browser__EventView__3(
        address_book, EventViewConfigurationFactory, DateTime, browser):
    """It renders a month headline only if days in this month are rendered."""
    EventViewConfigurationFactory(
        address_book, '1 week', start=0, duration=7)

    with mock.patch('icemac.ab.calendar.eventview.browser.date') as date:
        date.today.return_value = DateTime(2018, 2, 21).date()
        browser.login('cal-visitor')
        browser.lang(b'en')
        browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
    assert 'February 2018' in browser.ucontents
    assert 'March 2018' not in browser.ucontents


def test_browser__EventView__4(
        address_book, EventViewConfigurationFactory, DateTime, browser):
    """It renders sundays with a special css class."""
    EventViewConfigurationFactory(
        address_book, '1 week', start=0, duration=7)

    with mock.patch('icemac.ab.calendar.eventview.browser.date') as date:
        date.today.return_value = DateTime(2018, 2, 21).date()
        browser.login('cal-visitor')
        browser.lang(b'en')
        browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
    assert ' bg-info text-white">Sunday, 25.<' in browser.ucontents


def test_browser__EventView__5(
        address_book, EventViewConfigurationFactory, DateTime, browser):
    """It allows to select a different event view config."""
    EventViewConfigurationFactory(address_book, '1 week', start=0, duration=7)
    EventViewConfigurationFactory(
        address_book, '2 weeks', start=0, duration=14)

    with mock.patch('icemac.ab.calendar.eventview.browser.date') as date:
        date.today.return_value = DateTime(2018, 2, 21).date()
        browser.login('cal-visitor')
        browser.lang(b'en')
        browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
        assert 'March 2018' not in browser.ucontents

        browser.getControl(name='views:list').displayValue = ['2 weeks']
        browser.getForm().submit()
        assert 'March 2018' in browser.ucontents


def test_browser__EventView__7(
        address_book, EventViewConfigurationFactory, EventFactory,
        CategoryFactory, RecurringEventFactory, DateTime, browser):
    """It filters the events by the selected categories."""
    party = CategoryFactory(address_book, 'Party')
    EventFactory(address_book, category=party, datetime=DateTime.now)
    RecurringEventFactory(
        address_book, category=CategoryFactory(address_book, 'Lesson'),
        datetime=DateTime.add(DateTime.now, days=3),
        period='weekly')
    EventViewConfigurationFactory(
        address_book, '-1 day for 1 week', start=-1, duration=7,
        categories=set([party]))

    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
    assert '>Party<' in browser.ucontents
    assert '>Lesson<' not in browser.ucontents


def test_browser__EventView__8(
        address_book, EventViewConfigurationFactory, EventFactory,
        FieldFactory, PersonFactory, DateTime, browser):
    """It renders the selected fields."""
    field_name = FieldFactory(address_book, IEvent, 'Int', 'seats').__name__
    EventFactory(address_book, **{
        'datetime': DateTime.now,
        field_name: 42,
        'external_persons': ['Berta Vimladil'],
        'persons': set([PersonFactory(address_book, 'Tester')]),
        'text': 'to be canceled',
    })
    event_entity = IEntity(IEvent)
    EventViewConfigurationFactory(
        address_book, '-1 day for 1 week', start=-1, duration=7,
        fields=[event_entity.getRawField('persons'),
                event_entity.getRawField(field_name)])

    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_EVENT_VIEWS_URL)
    assert '>Berta Vimladil, Tester<' in browser.ucontents
    assert '>42<' in browser.ucontents
    assert 'to be canceled' not in browser.ucontents


def test_browser__EventView___get_events__1(
        address_book, EventViewConfigurationFactory, EventFactory,
        CategoryFactory, DateTime):
    """It returns all events in the configured interval."""
    cat = CategoryFactory(address_book, 'foo')
    EventFactory(
        address_book, category=cat, alternative_title='1',
        datetime=DateTime.now)
    EventFactory(
        address_book, category=cat, alternative_title='7',
        datetime=DateTime.add(DateTime.now, days=7))
    evc = EventViewConfigurationFactory(
        address_book, 'evc', start=0, duration=7, categories=set([cat]))
    view = EventView()
    view.context = address_book.calendar
    with zope.publisher.testing.interaction('zope.mgr'):
        events = view._get_events(evc)
    assert ['1', '7'] == [x.alternative_title for x in events]



from ..property import AddressBookField
import icemac.addressbook.addressbook


class ExampleSingle(object):
    """Example storing single values."""

    foo = AddressBookField('_storage')


class ExampleMultiple(object):
    """Example storing multiple values."""

    foo = AddressBookField('_storage', multiple=True)


def test_property__AddressBookField____get____1():
    """It returns itself on the class."""
    assert isinstance(ExampleSingle.foo, AddressBookField)


def test_property__AddressBookField____get____2():
    """It returns an empty tuple for multiple if no value got set."""
    assert () == ExampleMultiple().foo


def test_property__AddressBookField____get____3():
    """It returns `None` for single if no value got set."""
    assert ExampleSingle().foo is None


def test_property__AddressBookField____get____4(address_book):
    """It returns field objects for multiple values."""
    ex = ExampleMultiple()
    ex._storage = ('IcemacAbCalendarEventEvent###persons',
                   'IcemacAbCalendarEventEvent###category')
    assert (
        icemac.ab.calendar.event.event_entity.getRawField('persons'),
        icemac.ab.calendar.event.event_entity.getRawField('category'),
    ) == ex.foo


def test_property__AddressBookField____get____5(address_book):
    """It returns a single field object for a single value."""
    ex = ExampleSingle()
    ex._storage = ('IcemacAbCalendarEventEvent###persons', )
    assert (
        icemac.ab.calendar.event.event_entity.getRawField('persons') == ex.foo)


def test_property__AddressBookField____set____1(address_book):
    """It stores multiple values in a tuple on the instance."""
    ex = ExampleMultiple()
    ex.foo = [
        icemac.ab.calendar.event.event_entity.getRawField('persons'),
        icemac.ab.calendar.event.event_entity.getRawField('category'),
    ]
    assert ('IcemacAbCalendarEventEvent###persons',
            'IcemacAbCalendarEventEvent###category') == ex._storage


def test_property__AddressBookField____set____2(address_book):
    """It stores single values in a tuple on the instance."""
    ex = ExampleSingle()
    ex.foo = icemac.ab.calendar.event.event_entity.getRawField('category')
    assert ('IcemacAbCalendarEventEvent###category', ) == ex._storage


def test_property__AddressBookField____set____3(address_book):
    """It is able to set a single value to `None`."""
    ex = ExampleSingle()
    ex.foo = None
    assert () == ex._storage


def test_property__AddressBookField____get______set____1(address_book):
    """Get after set returns the values which were set. ."""
    ex = ExampleMultiple()
    value = (
        icemac.ab.calendar.event.event_entity.getRawField('persons'),
        icemac.ab.calendar.event.event_entity.getRawField('category'),
    )
    ex.foo = value
    assert value == ex.foo



import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.base


class View(icemac.addressbook.browser.base.BaseView):
    """Base for views in icemac.ab.calendar."""

    __PACKAGE_ID__ = icemac.ab.calendar.interfaces.PACKAGE_ID



from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.masterdata.breadcrumb
import icemac.addressbook.browser.base
import icemac.addressbook.browser.breadcrumb
import icemac.addressbook.browser.datamanager
import icemac.addressbook.browser.metadata
import icemac.addressbook.metadata.interfaces
import z3c.form.field
import zope.dublincore.interfaces
import zope.schema.interfaces


class MasterData(object):
    """List of master data of the calendar."""


class CalendarMasterDataBreadCrumb(
        icemac.addressbook.browser.breadcrumb.MasterdataChildBreadcrumb):
    """Breadcrumb for calendar master data."""

    grok.adapts(
        MasterData,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Calendar')


class ModifierMetadataGroup(
        icemac.addressbook.browser.metadata.MetadataBaseGroup):
    """Group displaying only meta data information of modifier.

    To be used when creator is empty and creation date is irrelevant.
    """

    fields = z3c.form.field.Fields(
        icemac.addressbook.metadata.interfaces.IEditor).select('modifier')
    fields += z3c.form.field.Fields(
        zope.dublincore.interfaces.IDCTimes).select('modified')


class CalendarView(icemac.addressbook.browser.base.GroupEditForm):
    """Edit the calendar view settings."""

    groups = (ModifierMetadataGroup,)
    interface = icemac.ab.calendar.interfaces.ICalendarDisplaySettings
    next_url = 'parent'


class CalendarEditBreadCrumb(
        icemac.ab.calendar.masterdata.breadcrumb.CalendarMDChildBreadcrumb):
    """Breadcrumb for calendar master data edit view."""

    grok.adapts(
        CalendarView,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _(u'Edit settings')
    target_url = None


class AnnotationField(icemac.addressbook.browser.datamanager.AnnotationField,
                      grok.MultiAdapter):
    """Special AnnotationField for calendar."""

    grok.adapts(icemac.ab.calendar.interfaces.ICalendar,
                zope.schema.interfaces.IField)

    @property
    def no_security_proxy(self):
        interfaces = [x[1] for x in zope.component.getUtilitiesFor(
            icemac.ab.calendar.interfaces.INoSecurityProxyType)]
        return interfaces



from icemac.addressbook.i18n import _
import icemac.addressbook.browser.addressbook.startpage


calendar = icemac.addressbook.browser.addressbook.startpage.StartpageData(
    'icemac.ab.calendar.interfaces.ICalendar', 'index.html', _('Calendar'))



from icemac.addressbook.browser.resource import base_css, form
import fanstatic
import icemac.ab.calendar.browser.interfaces
import icemac.addressbook.browser.favicon
import js.bootstrap4
import js.classy
import js.jquery
import zope.viewlet.viewlet


lib = fanstatic.Library('calendar', 'resources')
calendar_css = fanstatic.Resource(lib, 'calendar.css', depends=[base_css])
print_css = fanstatic.Resource(lib, 'print.css', depends=[calendar_css])
calendar_js = fanstatic.Resource(
    lib, 'calendar.js', depends=[
        js.jquery.jquery,
        js.classy.classy,
        form],
    bottom=True)


eventview_css = fanstatic.Resource(lib, 'eventview.css')
eventview_js = fanstatic.Resource(
    lib, 'eventview.js', depends=[js.bootstrap4.bootstrap])
eventview = fanstatic.Group([
    eventview_css,
    eventview_js
])


class CalendarResources(zope.viewlet.viewlet.ViewletBase):
    """Resources which are needed for the calendar."""

    def update(self):
        calendar_css.need()
        print_css.need()
        calendar_js.need()

    def render(self):
        return u''


def set_layer(context, request):
    """Set the calendar layer on the request, so the resources are rendered."""
    zope.interface.alsoProvides(
        request, icemac.ab.calendar.browser.interfaces.ICalendarLayer)


calendar_favicon = icemac.addressbook.browser.favicon.FavIconData(
    '/fanstatic/calendar/img/favicon.ico',
    '/fanstatic/calendar/img/favicon-preview.png')



# -*- coding: utf-8 -*-
from icemac.ab.calendar.interfaces import ICalendarDisplaySettings
from icemac.ab.calendar.interfaces import IEvent, IRecurringEvent
from icemac.ab.calendar.testing import CalendarWebdriverPageObjectBase
from icemac.ab.calendar.testing import get_recurred_event
from icemac.addressbook.testing import Webdriver
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait
from zope.traversing.browser import absoluteURL
import calendar
import pytest
import pytz
import zope.component.hooks


def test_event__Add__1(address_book, CategoryFactory, DateTime, browser):
    """It can add events which shown up in the calendar."""
    CategoryFactory(address_book, u'wedding day')
    dt = DateTime.today_8_32_am
    formatted_time = dt.strftime('%H:%M')
    formatted_date = DateTime.format_date(dt)
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getLink('event').click()
    assert browser.EVENT_ADD_URL == browser.url
    browser.getControl('date').value = formatted_date
    browser.getControl('time').value = formatted_time
    browser.getControl('event category').displayValue = ['wedding day']
    browser.getControl('Add', index=1).click()
    assert '"wedding day" added.' == browser.message
    assert browser.CALENDAR_MONTH_OVERVIEW_URL == browser.url
    # The new event shows up in the calendar:
    assert formatted_time in browser.contents


def test_event__Add__2(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.EVENT_ADD_URL)


def test_event__Edit__1(
        address_book, EventFactory, CategoryFactory, DateTime, browser):
    """It allows to edit an event."""
    CategoryFactory(address_book, u'wedding day')
    dt = DateTime.today_8_32_am
    EventFactory(address_book, datetime=dt)
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getLink('event', index=1).click()
    assert browser.EVENT_EDIT_URL == browser.url
    assert DateTime.format_date(dt) == browser.getControl('date').value
    assert dt.strftime('%H:%M') == browser.getControl('time').value
    browser.getControl('event category').displayValue = ['wedding day']
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    assert browser.CALENDAR_MONTH_OVERVIEW_URL == browser.url
    browser.getLink('wedding day').click()
    assert ['wedding day'] == browser.getControl('event category').displayValue


def test_event__Edit__2(address_book, EventFactory, DateTime, browser):
    """It is not accessible for a calendar visitor."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.login('cal-visitor')
    browser.open(browser.EVENT_EDIT_URL)
    # There are no fields to edit and no delete or clone button:
    assert (['form.buttons.apply', 'form.buttons.cancel'] ==
            browser.all_control_names)


def test_event__Edit__3(
        address_book, UserFactory, EventFactory, DateTime, browser):
    """It can be accessed by a local calendar user."""
    UserFactory(address_book, u'Hans', u'Tester', u'hans@example.com',
                '1qay2wsx', ['Calendar editor'])
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.formlogin('hans@example.com', '1qay2wsx')
    browser.open(browser.EVENT_EDIT_URL)
    assert len(browser.all_control_names) > 2


def test_event__Edit__4(
        address_book, EventFactory, PersonFactory, KeywordFactory,
        CategoryFactory, DateTime, browser):
    """It does not break if list of persons gets restricted by keyword.

    If a person is selected which does not have the person restriction
    keyword (see ICalendarDisplaySettings.person_keyword) the view does not
    break but renders the person as "missing".

    It renders an error message with the missing person selected.
    After deselecting the person the event can be saved again.
    """
    EventFactory(address_book,
                 datetime=DateTime.today_8_32_am,
                 category=CategoryFactory(address_book, u'foo'),
                 persons=set([PersonFactory(address_book, u'Tester')]))
    browser.login('cal-editor')
    browser.handleErrors = False
    browser.open(browser.EVENT_EDIT_URL)
    assert ['Tester'] == browser.getControl('persons').displayValue
    browser.getControl('notes').value = 'Force save'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    kw = KeywordFactory(address_book, u'foo')
    with zope.component.hooks.site(address_book):
        ICalendarDisplaySettings(address_book.calendar).person_keyword = kw
    browser.open(browser.EVENT_EDIT_URL)
    assert ['Missing: Tester'] == browser.getControl('persons').displayValue
    browser.getControl('notes').value = 'Force save, second time'
    browser.handleErrors = False
    browser.getControl('Apply').click()
    assert 'There were some errors.' in browser.contents
    assert ('Please deselect the persons prefixed with "Missing:"' in
            browser.contents)
    browser.getControl('persons').displayValue = []
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message


def test_event__Delete__1(address_book, EventFactory, DateTime, browser):
    """It allows to delete an event after a confirmation."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.login('cal-editor')
    browser.open(browser.EVENT_EDIT_URL)
    browser.getControl('Delete').click()
    assert 'Do you really want to delete this event?' in browser.contents
    assert browser.EVENT_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert '"event" deleted.' == browser.message
    assert browser.CALENDAR_MONTH_OVERVIEW_URL == browser.url


def test_event__Delete__2(
        address_book, EventFactory, CategoryFactory, DateTime, browser):
    """It renders a nice message if the event only has a category."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am,
                 category=CategoryFactory(address_book, u'foo'))
    browser.login('cal-editor')
    browser.open(browser.EVENT_DELETE_URL)
    browser.getControl('Yes').click()
    assert '"foo" deleted.' == browser.message


def test_event__Delete__3(address_book, EventFactory, DateTime, browser):
    """It is not accessible for a calendar visitor."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.EVENT_DELETE_URL)


def test_event__Clone__1(address_book, EventFactory, DateTime, browser):
    """It clones an event after a confirmation."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.login('cal-editor')
    browser.open(browser.EVENT_EDIT_URL)
    browser.getControl('Clone event').click()
    assert browser.EVENT_CLONE_URL == browser.url
    assert (['form.buttons.action', 'form.buttons.cancel'] ==
            browser.submit_control_names)
    browser.getControl('Yes').click()
    assert '"event" cloned.' == browser.message
    # Clone leads to edit view of cloned event:
    assert '{0.EVENT_EDIT_URL}-2'.format(browser) == browser.url


def test_event__Clone__2(address_book, EventFactory, DateTime, browser):
    """It is not accessible for a calendar visitor."""
    EventFactory(address_book, datetime=DateTime.today_8_32_am)
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.EVENT_CLONE_URL)


@pytest.fixture('function')
def sample_recurring_event(
        address_book, FieldFactory, CategoryFactory, RecurringEventFactory,
        PersonFactory, DateTime):
    """Example of a recurring event."""
    CategoryFactory(address_book, u'aaz')
    bar = CategoryFactory(address_book, u'bar')
    PersonFactory(address_book, u'Bester')
    tester = PersonFactory(address_book, u'Tester')
    FieldFactory(address_book, IEvent, u'Text', u'foobar')
    foobar = FieldFactory(
        address_book, IRecurringEvent, u'Text', u'foobar').__name__
    return RecurringEventFactory(
        address_book,
        **{'category': bar,
           foobar: u'qux',
           'datetime': DateTime(2014, 5, 24, 10, 30),
           'alternative_title': u'foo bär',
           'period': u'daily',
           'persons': set([tester]),
           'external_persons': [u'Mr. Developer'],
           'text': u'Important'})


def get_recurred_event_url(recurred_event, request):
    """Get the URL of a recurred event."""
    return absoluteURL(recurred_event, request).replace(
        'http://127.0.0.1/', 'http://localhost/')


@pytest.fixture('function')
def sample_recurred_event_url(
        sample_recurring_event, RequestFactory, DateTime):
    """Sample of a recurred event.

    Returns the URL to the @@customize-recurred-event view.
    """
    return get_recurred_event_url(
        get_recurred_event(sample_recurring_event, DateTime), RequestFactory())


def test_event__ViewRecurredEvent__1(
        address_book, sample_recurred_event_url, browser):
    """It renders a display form for a calendar visitor."""
    browser.login('cal-visitor')
    browser.open(sample_recurred_event_url)
    assert browser.RECURRED_EVENT_VIEW_URL == browser.url
    assert (['form.buttons.apply', 'form.buttons.cancel'] ==
            browser.all_control_names)


def test_event__AddFromRecurredEvent__1(
        address_book, DateTime, sample_recurred_event_url, browser):
    """It prefills the form from the recurring event."""
    browser.login('cal-editor')
    browser.open(sample_recurred_event_url)
    assert browser.RECURRED_EVENT_ADD_URL == browser.url
    assert ['bar'] == browser.getControl('event category').displayValue
    assert (DateTime.format_date(DateTime.now) ==
            browser.getControl('date').value)
    assert '10:30' == browser.getControl('time').value
    assert ('foo bär' ==
            browser.getControl('alternative title to category').value)
    assert ['Tester'] == browser.getControl('persons').displayValue
    assert ('Mr. Developer' ==
            browser.getControl(name='form.widgets.external_persons.0').value)
    assert 'Important' == browser.getControl('notes').value
    assert 'qux' == browser.getControl('foobar').value


def test_event__AddFromRecurredEvent__2(
        address_book, sample_recurred_event_url, browser):
    """It saves changes made in the form."""
    browser.login('cal-editor')
    browser.open(sample_recurred_event_url)
    browser.getControl('alternative title to category').value = 'birthday'
    browser.getControl('Apply').click()
    assert '"birthday" added.' == browser.message
    browser.open(browser.EVENT_EDIT_URL)
    assert 'birthday' == browser.getControl('alternative').value


def test_event__AddFromRecurredEvent__2_5(
        address_book, CategoryFactory, RecurringEventFactory, DateTime,
        RequestFactory, TimeZonePrefFactory, browser):
    """It keeps the time value in DST."""
    recurring_event = RecurringEventFactory(
        address_book,
        category=CategoryFactory(address_book, u'bar'),
        datetime=DateTime(2016, 3, 24, 12),
        period=u'weekly')
    recurred_event = recurring_event.get_events(
        DateTime(2016, 3, 27, 0), DateTime(2016, 4, 1, 0), pytz.utc).next()
    url = get_recurred_event_url(recurred_event, RequestFactory())
    TimeZonePrefFactory('Europe/Berlin')
    browser.login('cal-editor')
    browser.open(url)
    # Without making sure the local time stays the same also in DST the time
    # would be 14:00 because in DST Berlin is 2 hours ahead of UTC.
    assert '13:00' == browser.getControl('time').value


def test_event__AddFromRecurredEvent__3(
        address_book, sample_recurred_event_url, browser):
    """It does not change anything if `cancel` gets hit."""
    browser.login('cal-editor')
    browser.open(sample_recurred_event_url)
    browser.getControl('Cancel').click()
    assert 'Addition canceled.' == browser.message
    assert 0 == len(address_book.calendar)


def test_event__AddFromRecurredEvent__4(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.RECURRED_EVENT_ADD_URL)


def days_in_month(date):
    """Number of days in the month `date` belongs to."""
    return calendar.monthrange(date.year, date.month)[1]


def test_event__DeleteRecurredEvent__1(
        address_book, sample_recurred_event_url, DateTime, browser):
    """It deletes the recurred event after a confirmation."""
    browser.login('cal-editor')
    browser.open(sample_recurred_event_url)
    browser.getControl('Delete').click()
    assert browser.RECURRED_EVENT_DELETE_URL == browser.url
    assert ('Do you really want to delete this recurred event?' in
            browser.contents)
    browser.getControl('Yes').click()
    assert u'"foo bär" deleted.' == browser.message
    days = days_in_month(DateTime.now)  # -1 deleted event +1 browser.message
    assert days == browser.contents.count('foo bär')
    assert address_book.calendar['Event'].deleted


def test_event__DeleteRecurredEvent__1_5(
        address_book, sample_recurring_event, sample_recurred_event_url,
        DateTime, browser):
    """It deletes a whole day recurred event, too."""
    sample_recurring_event.whole_day_event = True
    browser.login('cal-editor')
    browser.open(sample_recurred_event_url)
    browser.getControl('Delete').click()
    browser.getControl('Yes').click()
    assert u'"foo bär" deleted.' == browser.message
    days = days_in_month(DateTime.now)  # -1 deleted event +1 browser.message
    assert days == browser.contents.count('foo bär')


def test_event__DeleteRecurredEvent__2(
        address_book, sample_recurring_event, RequestFactory, DateTime,
        browser):
    """It renders a nice message if the recurring event has only a category."""
    sample_recurring_event.alternative_title = None
    browser.login('cal-editor')
    browser.open(get_recurred_event_url(
        get_recurred_event(sample_recurring_event, DateTime),
        RequestFactory()))
    browser.getControl('Delete').click()
    browser.getControl('Yes').click()
    assert u'"bar" deleted.' == browser.message


def test_event__DeleteRecurredEvent__3(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.RECURRED_EVENT_DELETE_URL)


@pytest.fixture('function')
def sample_event_webdriver(address_book, EventFactory, DateTime):
    """Create a sample event."""
    return EventFactory(address_book, datetime=DateTime.today_8_32_am)


class POEvent(CalendarWebdriverPageObjectBase):
    """Webdriver page object for an event object itself."""

    paths = [
        'EVENT_EDIT_URL',
    ]

    # property setter!
    def set_whole_day_event(self, value):
        self._selenium.find_element_by_id(
            'form-widgets-datetime-widgets-whole_day_event-{}'.format(
                int(not(value)))).click()

    whole_day_event = property(None, set_whole_day_event)

    def _assert_element_hidden(self, el):
        assert not el.is_displayed()

    def _assert_element_shown(self, el):
        assert el.is_displayed()

    def _wait_for_element_hidden(self, el):
        WebDriverWait(self._selenium, 3).until_not(
            expected_conditions.visibility_of(el))

    def _wait_for_element_shown(self, el):
        WebDriverWait(self._selenium, 3).until(
            expected_conditions.visibility_of(el))

    status_attr_map = {
        'hidden': _assert_element_hidden,
        'shown': _assert_element_shown,
        'wait-for-hidden': _wait_for_element_hidden,
        'wait-for-shown': _wait_for_element_shown,
    }

    def assert_time(self, status):
        """Assert display status of time widget.

        Possible status see keys of `status_attr_map`.
        """
        el = self._selenium.find_element_by_id(
            'form-widgets-datetime-widgets-time')
        return self.status_attr_map[status](self, el)


Webdriver.attach(POEvent, 'event')


@pytest.mark.webdriver
def test_event__WidgetToggle__1_webdriver(sample_event_webdriver, webdriver):
    """The time widget is initially hidden for whole day events."""
    sample_event_webdriver.whole_day_event = True
    webdriver.login('cal-editor', webdriver.event.EVENT_EDIT_URL)
    webdriver.event.assert_time('hidden')


@pytest.mark.webdriver
def test_event__WidgetToggle__2_webdriver(sample_event_webdriver, webdriver):
    """The time widget is initially shown for non whole day events."""
    sample_event_webdriver.whole_day_event = False
    webdriver.login('cal-editor', webdriver.event.EVENT_EDIT_URL)
    webdriver.event.assert_time('shown')


@pytest.mark.webdriver
def test_event__WidgetToggle__3_webdriver(sample_event_webdriver, webdriver):
    """Changing an event to a whole day event hides the time widget."""
    sample_event_webdriver.whole_day_event = False
    webdriver.login('cal-editor', webdriver.event.EVENT_EDIT_URL)
    webdriver.event.whole_day_event = True
    webdriver.event.assert_time('wait-for-hidden')


@pytest.mark.webdriver
def test_event__WidgetToggle__4_webdriver(sample_event_webdriver, webdriver):
    """Changing an event to a non-whole day event shows the time widget."""
    sample_event_webdriver.whole_day_event = True
    webdriver.login('cal-editor', webdriver.event.EVENT_EDIT_URL)
    webdriver.event.whole_day_event = False
    webdriver.event.assert_time('wait-for-shown')


@pytest.mark.webdriver
def test_event__WidgetToggle__5_webdriver(sample_event_webdriver, webdriver):
    """Clicking on a selected event kind does not toggle the time display."""
    sample_event_webdriver.whole_day_event = True
    webdriver.login('cal-editor', webdriver.event.EVENT_EDIT_URL)
    webdriver.event.assert_time('hidden')
    webdriver.event.whole_day_event = True
    webdriver.event.assert_time('wait-for-hidden')



import icemac.ab.calendar.interfaces
import icemac.addressbook.fieldsource
import icemac.addressbook.interfaces


class AddressBookField(object):
    """Property which (de-)serializes a field object from/to a token.

    Caution: This class only works for fields on the event entity!
    """

    def __init__(self, attribute_name, multiple=False):
        self.multiple = multiple
        self.attrib = attribute_name

    def __get__(self, instance, class_):
        """Untokenize the stored value(s) to field objects."""
        if instance is None:
            return self
        values = []
        for token in getattr(instance, self.attrib, ()):
            try:
                field = icemac.addressbook.fieldsource.untokenize(token)[1]
            except KeyError:
                pass
            else:
                values.append(field)
        if self.multiple:
            return tuple(values)
        return values[0] if values else None

    def __set__(self, instance, value):
        """Store field objects as tokens."""
        if not self.multiple:
            value = [value] if value else []
        event_entity = icemac.addressbook.interfaces.IEntity(
            icemac.ab.calendar.interfaces.IEvent)
        tokens = [
            icemac.addressbook.fieldsource.tokenize(
                event_entity, field.__name__)
            for field in value]

        setattr(instance, self.attrib, tuple(tokens))



# -*- coding: utf-8 -*-
from .interfaces import IEventViewConfiguration
from .interfaces import IEventViewContainer
from .model import EventViewConfiguration
from .model import EventViewContainer
from zope.interface.verify import verifyObject


def test_model__EventViewContainer__1():
    """It it fulfils the IEventViewContainer interface."""
    assert verifyObject(IEventViewContainer, EventViewContainer())


def test_model__EventViewConfiguration__1():
    """It it fulfils the IEventViewConfiguration interface."""
    assert verifyObject(IEventViewConfiguration, EventViewConfiguration())


def test_model__EventViewConfiguration____repr____1():
    """It has a human readable `repr` even if no title is set."""
    evc = EventViewConfiguration()
    evc.title = u'Have a 🧠!'
    assert isinstance(repr(evc), str)
    assert repr(evc) == "<EventViewConfiguration title=u'Have a \\U0001f9e0!'>"


def test_model__EventViewConfiguration____repr____2():
    """It has a human readable `repr` even if no title is set."""
    evc = EventViewConfiguration()
    assert repr(evc) == '<EventViewConfiguration title=None>'



from icemac.ab.calendar.interfaces import IEvent
from icemac.ab.calendar.interfaces import IRecurringEvent
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.menus.menu
import z3c.menu.ready2go.checker
import z3c.menu.ready2go.item
import zope.interface


class CalendarMenuItem(z3c.menu.ready2go.item.SiteMenuItem):
    """Menu item for the calendar tab in the site menu."""


class CalendarMenuItemSelectedChecker(
        z3c.menu.ready2go.checker.TrueSelectedChecker,
        grok.MultiAdapter):
    """Selected checker for the calendar menu item in the site menu."""

    grok.adapts(zope.interface.Interface,
                icemac.addressbook.browser.interfaces.IAddressBookLayer,
                zope.interface.Interface,
                icemac.addressbook.browser.menus.menu.MainMenu,
                CalendarMenuItem)

    CALENDAR_VIEW_NAMES = (
        'addEvent.html',
        'addFromRecurredEvent.html',
        'month.html',
        'year.html',
    )

    @property
    def selected(self):
        if IEvent.providedBy(self.context):
            return not(IRecurringEvent.providedBy(self.context))
        if self.view.__name__ in self.CALENDAR_VIEW_NAMES:
            return True
        return False



import icemac.addressbook.generations.utils
import zope.component
import zope.catalog.interfaces
from ..interfaces import DATE_INDEX, IEventDateTime


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Do not catalog recurring events."""
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    index = catalog.get(DATE_INDEX)
    index.interface = IEventDateTime



from icemac.addressbook.i18n import _
import gocept.reference
import icemac.ab.calendar.eventview.interfaces
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.property
import icemac.addressbook.utils
import persistent
import zope.container.btree
import zope.container.contained
import zope.interface
import zope.schema.fieldproperty


@zope.interface.implementer(
    icemac.ab.calendar.eventview.interfaces.IEventViewContainer)
class EventViewContainer(zope.container.btree.BTreeContainer):
    """A container for event view configurations."""


@zope.interface.implementer(
    icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration)
class EventViewConfiguration(persistent.Persistent,
                             zope.container.contained.Contained):
    """A configuration of an view of events."""

    zope.schema.fieldproperty.createFieldProperties(
        icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration,
        omit=['categories', 'fields'])
    categories = gocept.reference.ReferenceCollection(
        'categories', ensure_integrity=True)
    fields = icemac.ab.calendar.property.AddressBookField(
        '_fields', multiple=True)

    def __init__(self, *args, **kw):
        super(EventViewConfiguration, self).__init__(*args, **kw)
        self.categories = set([])

    def __repr__(self):
        """Human readable representation of the object."""
        return u'<EventViewConfiguration title={0!r}>'.format(self.title)


unique_titles = icemac.addressbook.utils.unique_by_attr_factory(
    'title', _('This title is already used for an event view configuration.'))



import icemac.addressbook.generations.utils


ICalendar = 'icemac.ab.calendar.interfaces.ICalendar'


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Fix calendar start page to have a non-None view name."""
    if addressbook.startpage == (ICalendar, None):
        addressbook.startpage = (ICalendar, 'index.html')



from __future__ import unicode_literals
from datetime import date  # be able to mock date.today() in tests
from icemac.ab.calendar.browser.renderer.table import get_day_names
from icemac.ab.calendar.browser.renderer.table import render_event_time
from icemac.ab.calendar.browser.resource import eventview
from icemac.ab.calendar.eventview.interfaces import IEventViews
from icemac.ab.calendar.eventview.interfaces import IEventViewConfiguration
from icemac.addressbook.preferences.utils import get_time_zone_name
import copy
import datetime
import grokcore.component as grok
import icemac.ab.calendar.browser.base
import icemac.ab.calendar.browser.calendar
import icemac.ab.calendar.eventview.interfaces
import pytz
import z3c.form.interfaces
import zope.cachedescriptors.property
import zope.component


ONE_DAY = datetime.timedelta(days=1)


class EventData(icemac.ab.calendar.browser.calendar.EventDescriptionBase,
                grok.MultiAdapter):
    """Adapter from Event to IEventData needed by renderer."""

    grok.adapts(icemac.ab.calendar.interfaces.IBaseEvent,
                IEventViewConfiguration)
    grok.implements(icemac.ab.calendar.eventview.interfaces.IEventData)

    def __init__(self, event, config):
        super(EventData, self).__init__(event)
        self.config = config
        self.title = self._text
        additional_event_fields = copy.copy(config.fields)
        self.fields = self._get_info_from_fields(additional_event_fields)


class EventList(list):
    """List of events."""

    def pop(self, default=None):
        try:
            return super(EventList, self).pop()
        except IndexError:
            return default


class EventView(icemac.ab.calendar.browser.base.View):
    """Render the views configured as event view configurations."""

    start = None
    end = None
    events = None

    def update(self):
        eventview.need()
        self.widget = zope.component.getMultiAdapter(
            (IEventViews['views'], self.request),
            z3c.form.interfaces.IFieldWidget)
        self.widget.update()
        if 'views' in self.request.form:
            # The used select widget is a multi select (which cannot be
            # changed):
            token = self.widget.value[0]
            self.event_view_config = self.widget.terms.getValue(token)
        else:
            source = IEventViews['views'].source.factory
            possible_event_view_configs = source.getValues()
            if possible_event_view_configs:
                self.event_view_config = possible_event_view_configs[0]
            else:
                self.event_view_config = None
        self.events = EventList(self._get_events(self.event_view_config))
        self.events.reverse()

    def close_url(self):
        return self.url(self.context, to='month')

    def action(self):
        return self.url(self.context, self.__name__)

    def views(self):
        self.widget.klass = "form-control"
        return self.widget.render()

    @zope.cachedescriptors.property.Lazy
    def month_names(self):
        """Return a list of month names (one based!) in locale of user."""
        locale_calendar = self.request.locale.dates.calendars['gregorian']
        return ['NULL'] + locale_calendar.getMonthNames()

    @zope.cachedescriptors.property.Lazy
    def day_names(self):
        return get_day_names(self.request)

    def events_per_month(self):
        month = {
            'month': '{} {}'.format(
                self.month_names[self.start.month], self.start.year),
            'days': [],
        }
        current_day = self.start
        current_event = self.events.pop(default=None)
        while current_day <= self.end:
            current_week_day = int(current_day.strftime('%w'))
            add_css_dt = []
            add_css_dd = ''
            if current_week_day == 0:
                add_css_dt.append(' bg-info text-white')
                add_css_dd = ' bg-info text-white'
                css_a = 'text-white'
            else:
                css_a = 'text-dark'
            day = {
                'day': '{}, {}.'.format(
                    self.day_names[current_week_day], current_day.day),
                'add_css_dt': ''.join(add_css_dt),
                'add_css_dd': add_css_dd,
                'css_a': css_a,
                'events': [],
            }
            next_day = current_day + ONE_DAY
            while (current_event is not None
                   and current_event.datetime < next_day):
                ed = zope.component.getMultiAdapter(
                    (current_event, self.event_view_config),
                    icemac.ab.calendar.eventview.interfaces.IEventData)
                event_data = {
                    'event': ed.title,
                    'url': self.url(current_event),
                    'time': render_event_time(
                        ed.datetime, ed.whole_day, self.request),
                    'data': ed.fields,
                }
                day['events'].append(event_data)

                current_event = self.events.pop(default=None)

            month['days'].append(day)
            if next_day.day == 1:
                yield month
                month = {
                    'month': '{} {}'.format(
                        self.month_names[next_day.month], next_day.year),
                    'days': [],
                }
            current_day = next_day
        if month['days']:
            yield month

    def _get_events(self, event_view_config):
        """Get events between `start` and `duration` of EventViewConfig."""
        if not event_view_config:
            return []
        midnight = datetime.time(0, 0, 0, tzinfo=pytz.UTC)
        today = date.today()
        self.start = datetime.datetime.combine(
            today + datetime.timedelta(days=event_view_config.start),
            midnight)
        self.end = self.start + datetime.timedelta(
            days=event_view_config.duration + 1, seconds=-1)
        categories = [x.title for x in event_view_config.categories]
        return self.context.get_events(
            self.start, self.end, get_time_zone_name(), categories)



import icemac.addressbook.browser.about.about


class CalendarIcon(icemac.addressbook.browser.about.about.IconProviderInfo):
    name = u'GraphicsFuel'
    url = u'http://www.graphicsfuel.com/2012/02/calendar-icon-psd'



from ..interfaces import PersonSource, ICalendarDisplaySettings, ICalendar
from icemac.addressbook.interfaces import ITitle, IKeywords
import zope.component


pytest_plugins = 'icemac.addressbook.browser.conftest'


def test_interfaces__PersonSource__getValue__1(address_book, search_data):
    """It returns all persons in address book if no `person_keyword` is set."""
    assert ([u'Hohmuth', u'Koch', u'Velleuer', u'Liebig', u'Tester, Liese'] ==
            [ITitle(x) for x in PersonSource().factory.getValues()])


def test_interfaces__PersonSource__getValue__2(address_book, search_data):
    """It returns only the persons in address book having `person_keyword`."""
    kw = zope.component.getUtility(IKeywords).get_keyword_by_title('church')
    # `search_data` yields an address_book instance which contains the person
    # data and is set as site, so we have to use it here:
    ICalendarDisplaySettings(ICalendar(search_data)).person_keyword = kw
    assert ([u'Koch', u'Velleuer', u'Liebig'] ==
            [ITitle(x) for x in PersonSource().factory.getValues()])



from gocept.reference.verify import verifyObject
from icemac.ab.calendar.event import BaseEvent, Event, RecurredEvent
from icemac.ab.calendar.event import RecurringEvent, RecurringEventContainer
from icemac.ab.calendar.event import _get_field_name_on_IEvent
from icemac.ab.calendar.event import get_event_data_from_recurring_event
from icemac.ab.calendar.interfaces import IEvent, IEventDateTime, ICalendar
from icemac.ab.calendar.interfaces import IRecurredEvent, DATE_INDEX
from icemac.ab.calendar.interfaces import IRecurringEvent, IRecurringEvents
from icemac.addressbook.interfaces import IEntity, ITitle, ISchemaName
import pytz
import zope.catalog.interfaces
import zope.component


def test_event__BaseEvent__listPersons__1(address_book, PersonFactory):
    """It returns a list of persons in the address book and externals."""
    event = BaseEvent()
    event.persons = set([
        PersonFactory(address_book, u'Tester', first_name=u'Hans'),
        PersonFactory(address_book, u'Koch', first_name=u'Fritz')])
    event.external_persons = [u'Klaus Arkpe', u'Heiner Myer']
    assert ([u'Fritz Koch', u'Hans Tester', u'Heiner Myer', u'Klaus Arkpe'] ==
            event.listPersons())


def test_event__BaseEvent__in_timezone__1(DateTime):
    """It returns a whole day event as midnight of the given time zone."""
    tz = pytz.timezone('Europe/Berlin')
    event = BaseEvent()
    event.whole_day_event = True
    event.datetime = DateTime(2015, 1, 5, 13, 13)
    assert DateTime(2015, 1, 5, 0, tzinfo=tz) == event.in_timezone(tz)


def test_event__BaseEvent__in_timezone__2(DateTime):
    """It returns the datetime of a non-whole-day event in given time zone."""
    tz = pytz.timezone('Europe/Berlin')
    event = BaseEvent()
    event.whole_day_event = False
    event.datetime = DateTime(2015, 1, 5, 10)  # UTC
    assert tz.normalize(DateTime(2015, 1, 5, 10)) == event.in_timezone(tz)


def test_event__Event__1(zcmlS):
    """It implements the `IEvent` interface."""
    assert verifyObject(IEvent, Event())


def test_event__Event__2(
        address_book, FullPersonFactory, EventFactory, browser):
    """A person referenced on an event can still become a principal."""
    person = FullPersonFactory(
        address_book, u'Tester', email__email=u'tester@example.com')
    EventFactory(address_book, persons=set([person]))
    browser.login('mgr')
    browser.open(browser.PRINCIPAL_ADD_URL)
    # A user referenced on an event is still in the list of persons which
    # might become a principal:
    assert ['Tester'] == browser.getControl('person').displayOptions


def test_event__Event__3(address_book, CategoryFactory, DateTime, browser):
    """A new field can be added to it and it can be used."""
    CategoryFactory(address_book, u'Wedding')
    browser.login('mgr')
    browser.open(browser.ENTITIES_EDIT_URL)
    # It is possible to a a new field to an event:
    browser.getLink('Edit fields', index=8).click()
    assert browser.CALENDAR_EVENT_FIELDS_LIST_URL == browser.url
    browser.getLink('field').click()
    browser.getControl('type').displayValue = ['integer number']
    browser.getControl('title').value = 'Number of reservations'
    browser.getControl('Add', index=1).click()
    assert '"Number of reservations" added.' == browser.message
    # This new field can be used in the event add form:
    browser.open(browser.EVENT_ADD_URL)
    browser.getControl('date').value = DateTime.format_date(DateTime.now)
    browser.getControl('yes').selected = True  # whole day event?
    browser.getControl('event category').displayValue = ['Wedding']
    browser.getControl('Number of reservations').value = '42'
    browser.getControl('Add', index=1).click()
    assert '"Wedding" added.' == browser.message
    # and it can be used in the event edit form:
    browser.open(browser.EVENT_EDIT_URL)
    assert '42' == browser.getControl('Number of reservations').value
    browser.getControl('Number of reservations').value = '41'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message


def test_event__Event____repr____1(zcmlS, DateTime):
    """It returns some data of the event."""
    event = Event()
    event.datetime = DateTime(2016, 4, 6, 16)
    event.alternative_title = u'my-title'
    assert (
        "<Event datetime='2016-04-06 16:00:00+00:00' title=u'my-title', "
        "deleted=False>" == repr(event))


def test_event__Event__schema__1(zcmlS):
    """It can be adapted to ``ISchemaName``."""
    assert 'IEvent' == ISchemaName(Event()).schema_name


def test_event__RecurringEventContainer__1():
    """It implements the `IRecurringEvents` interface."""
    assert verifyObject(IRecurringEvents, RecurringEventContainer())


def test_event__RecurringEventContainer__get_events__1(
        address_book, RecurringEventFactory, DateTime):
    """It sorts the events by their weights."""
    RecurringEventFactory(
        address_book,
        datetime=DateTime.now,
        alternative_title=u'weekly',
        period='weekly')
    RecurringEventFactory(
        address_book,
        datetime=DateTime.now,
        alternative_title=u'yearly',
        period='yearly')
    RecurringEventFactory(
        address_book,
        datetime=DateTime.now,
        alternative_title=u'biweekly',
        period='biweekly')
    RecurringEventFactory(
        address_book,
        datetime=DateTime.now,
        alternative_title=u'nth weekday of month',
        period='nth weekday of month')
    recurring_events = zope.component.getUtility(IRecurringEvents)
    assert ([u'weekly', u'biweekly', u'nth weekday of month', u'yearly'] ==
            [x.alternative_title for x in recurring_events.get_events([])])


def test_event__RecurringEvent__1(zcmlS, DateTime):
    """It implements the `IRecurringEvent` interface."""
    revent = RecurringEvent()
    revent.datetime = DateTime.now
    revent.period = u'weekly'
    assert verifyObject(IRecurringEvent, revent)


def test_event__RecurringEvent____repr____1(zcmlS, DateTime):
    """It returns some data of the recurring event."""
    event = RecurringEvent()
    event.datetime = DateTime(2016, 4, 6, 16)
    event.alternative_title = u'my-rec-title'
    assert (
        "<RecurringEvent datetime='2016-04-06 16:00:00+00:00' "
        "title=u'my-rec-title', deleted=False>" == repr(event))


def test_event__RecurringEvent__schema__1(zcmlS):
    """It can be adapted to ``ISchemaName``."""
    assert 'IRecurringEvent' == ISchemaName(RecurringEvent()).schema_name


def test_event__RecurringEvent__get_events__1(
        address_book, RecurringEventFactory, CategoryFactory, DateTime):
    """It returns an iterable of `RecurredEvent` instances."""
    recurring_event = RecurringEventFactory(
        address_book,
        datetime=DateTime(2014, 5, 2, 12),
        text=u'foobar',
        period='weekly',
        category=CategoryFactory(address_book, u'birthday'))
    events = list(recurring_event.get_events(
        DateTime(2014, 5, 1, 0), DateTime(2014, 5, 8, 0), pytz.utc))
    assert 1 == len(events)
    event = events[0]
    assert isinstance(event, RecurredEvent)
    assert 'foobar' == event.text


def test_event__RecurringEvent__get_events__2(
        address_book, RecurringEventFactory, CategoryFactory, DateTime):
    """It ends the recurrence at the `end` date."""
    recurring_event = RecurringEventFactory(
        address_book,
        datetime=DateTime(2014, 4, 18, 12),
        end=DateTime(2014, 5, 9, 0).date(),
        period='weekly',
        category=CategoryFactory(address_book, u'event'))
    events = list(recurring_event.get_events(
        DateTime(2014, 5, 1, 0), DateTime(2014, 5, 31, 0), pytz.utc))
    assert 2 == len(events)
    assert ([DateTime(2014, 5, 2, 12),
             DateTime(2014, 5, 9, 12)] == [x.datetime for x in events])

    # If the end date is not reached all recurrences until the requested
    # interval end are returned:
    events = list(recurring_event.get_events(
        DateTime(2014, 4, 1, 0), DateTime(2014, 4, 30, 0), pytz.utc))
    assert 2 == len(events)
    assert ([DateTime(2014, 4, 18, 12),
             DateTime(2014, 4, 25, 12)] == [x.datetime for x in events])


def test_event__RecurringEvent__get_events__3(
        address_book, RecurringEventFactory, CategoryFactory, DateTime):
    """The local time of the recurred event does not change in DST.

    DST ... daylight saving time
    """
    recurring_event = RecurringEventFactory(
        address_book,
        datetime=DateTime(2016, 3, 24, 12),
        period='weekly',
        category=CategoryFactory(address_book, u'event'))
    tz_berlin = pytz.timezone('Europe/Berlin')
    # At 2016-03-27 DST starts in Europe/Berlin
    events = list(recurring_event.get_events(
        DateTime(2016, 3, 24, 0), DateTime(2016, 4, 1, 0), tz_berlin))
    assert ([DateTime(2016, 3, 24, 13, tzinfo=tz_berlin),
             DateTime(2016, 3, 31, 13, tzinfo=tz_berlin)] ==
            [x.datetime for x in events])
    # So the time in UTC changes to keep it the same in local time:
    assert ([DateTime(2016, 3, 24, 12),
             DateTime(2016, 3, 31, 11)] ==
            [pytz.utc.normalize(x.datetime) for x in events])


def test_event__RecurredEvent__create_from__1(
        address_book, FieldFactory, RecurringEventFactory, CategoryFactory,
        PersonFactory, DateTime):
    """It copies attributes from the given parameters."""
    event_field_name = FieldFactory(
        address_book, IEvent, u'Int', u'reserve').__name__
    revent_field = FieldFactory(
        address_book, IRecurringEvent, u'Int', u'reserve').__name__
    data = {
        'datetime': DateTime(2014, 5, 2, 12),
        'category': CategoryFactory(address_book, u'birthday'),
        'period': 'weekly',
        'persons': set([PersonFactory(address_book, u'Tester')]),
        revent_field: 42}
    recurring_event = RecurringEventFactory(address_book, **data)

    recurred_event = RecurredEvent.create_from(
        recurring_event, DateTime(2014, 4, 12, 21))
    assert DateTime(2014, 4, 12, 21) == recurred_event.datetime
    assert list(recurring_event.persons)[0] in recurred_event.persons
    assert recurring_event.category == recurred_event.category
    assert address_book.calendar == recurred_event.__parent__
    assert recurring_event == recurred_event.recurring_event
    assert 42 == getattr(recurred_event, event_field_name)


def test_event__RecurringEvent__2(
        address_book, EventFactory, RecurringEventFactory, DateTime):
    """It does not get catalogued but only Event objects."""
    dt = DateTime.now
    event = EventFactory(address_book, datetime=dt)
    RecurringEventFactory(address_book, datetime=dt, period='weekly')
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    results = catalog.searchResults(**{DATE_INDEX: {'any': None}})
    assert [event] == list(results)


def test_event__RecurredEvent__1():
    """It implements the `IRecurredEvent` interface."""
    assert verifyObject(IRecurredEvent, RecurredEvent())


def test_event__RecurredEvent____repr____1(
        address_book, CategoryFactory, RecurringEventFactory, DateTime):
    """It returns some data of the recurred event."""
    recurring_event = RecurringEventFactory(
        address_book,
        datetime=DateTime(2014, 5, 2, 12),
        period='weekly',
        category=CategoryFactory(address_book, u'birthday'))
    events = list(recurring_event.get_events(
        DateTime(2014, 5, 1, 0), DateTime(2014, 5, 8, 0), pytz.utc))
    recurred_event = events[0]
    assert isinstance(recurred_event, RecurredEvent)
    assert ("<RecurredEvent datetime='2014-05-02 12:00:00+00:00' "
            "title=u'birthday'>" == repr(recurred_event))


def test_event__EventDateTime__1(address_book, EventFactory, DateTime):
    """A whole day event gets indexed as noon in timzone utc."""
    event = EventFactory(address_book,
                         datetime=DateTime(2015, 1, 5, 14, 14),
                         whole_day_event=True)
    adapter = IEventDateTime(event)
    assert DateTime(2015, 1, 5, 12) == adapter.datetime
    assert pytz.utc == adapter.datetime.tzinfo


def test_event___get_field_name_on_IEvent__1(address_book, FieldFactory):
    """It only returns the field name if the title type and the value match ...

    between event and recurring event.
    """
    event_field = FieldFactory(address_book, IEvent, u'Bool', u'foo')
    revent_field = FieldFactory(address_book, IRecurringEvent, u'Bool', u'foo')
    assert (event_field.__name__ ==
            _get_field_name_on_IEvent(revent_field, IEntity(IEvent)))


def test_event___get_field_name_on_IEvent__2(address_book, FieldFactory):
    """It returns `None` if the fields do not match exactly."""
    FieldFactory(address_book, IEvent, u'Choice', u'foo', values=[u'a'])
    revent_field = FieldFactory(
        address_book, IRecurringEvent, u'Choice', u'foo', values=[u'a', u'b'])
    assert None is _get_field_name_on_IEvent(revent_field, IEntity(IEvent))


def test_event___get_field_name_on_IEvent__3(zcmlS):
    """It returns the field name for a field defined on `IBaseEvent`."""
    assert 'category' == _get_field_name_on_IEvent(IRecurringEvent['category'],
                                                   IEntity(IEvent))


def test_event___get_field_name_on_IEvent__4(zcmlS):
    """It returns `None` for a field not defined on `IBaseEvent`."""
    assert None is _get_field_name_on_IEvent(IRecurringEvent['period'],
                                             IEntity(IEvent))


def test_event__get_event_data_from_recurring_event__1(
        address_book, CategoryFactory, RecurringEventFactory, DateTime):
    """It returns a dict of event data and datetime."""
    category = CategoryFactory(address_book, u'bar')
    recurring_event = RecurringEventFactory(
        address_book,
        category=category,
        period='weekly',
        datetime=DateTime(2014, 5, 24, 10, 30),
        alternative_title=u'foo bar')
    assert ({
        'alternative_title': u'foo bar',
        'category': category,
        'datetime': DateTime(2000, 1, 1, 10, 30),
        'external_persons': None,
        'persons': None,
        'text': None,
        'whole_day_event': False} == get_event_data_from_recurring_event(
            recurring_event, DateTime(2000, 1, 1, 10, 30)))


def test_event__get_event_data_from_recurring_event__2(
        address_book, CategoryFactory, RecurringEventFactory, DateTime):
    """It returns whole day events."""
    category = CategoryFactory(address_book, u'bar')
    recurring_event = RecurringEventFactory(
        address_book,
        category=category,
        period='weekly',
        datetime=DateTime(2014, 5, 24, 1),
        whole_day_event=True,
        alternative_title=u'foo bar')
    assert ({
        'alternative_title': u'foo bar',
        'category': category,
        'datetime': DateTime(2000, 1, 1, 1),
        'external_persons': None,
        'persons': None,
        'text': None,
        'whole_day_event': True} == get_event_data_from_recurring_event(
            recurring_event, DateTime(2000, 1, 1, 1)))


def test_event__get_event_data_from_recurring_event__3(
        address_book, FieldFactory, RecurringEventFactory, DateTime):
    """It returns the appropriate user defined fields."""
    revent_foo = FieldFactory(
        address_book, IRecurringEvent, u'Text', u'foo').__name__
    revent_bar = FieldFactory(
        address_book, IRecurringEvent, u'Text', u'bar').__name__
    FieldFactory(address_book, IEvent, u'Text', u'foo')
    recurring_event = RecurringEventFactory(
        address_book, **{
            revent_foo: u'asdf',
            revent_bar: u'qwe',
            'period': 'weekly',
            'datetime': DateTime(2014, 5, 24, 10, 30)})
    assert ({
        'Field-3': 'asdf',
        'alternative_title': None,
        'category': None,
        'datetime': DateTime(2000, 1, 1, 10, 30),
        'external_persons': None,
        'persons': None,
        'text': None,
        'whole_day_event': False} == get_event_data_from_recurring_event(
            recurring_event, DateTime(2000, 1, 1, 10, 30)))


def test_event__title__1(address_book, EventFactory):
    """It returns the alternative title if it is set."""
    event = EventFactory(address_book, alternative_title=u'alt-title')
    assert u'alt-title' == ITitle(event)


def test_event__title__2(address_book, CategoryFactory, EventFactory):
    """It returns the category title if the alternative title is not set."""
    event = EventFactory(
        address_book, category=CategoryFactory(address_book, u'birthday'))
    assert u'birthday' == ITitle(event)


def test_event__title__3(address_book, EventFactory):
    """It returns a string if neither alt. title nor category are set."""
    assert u'event' == ITitle(EventFactory(address_book))


def test_event__get_calendar__1(address_book, EventFactory):
    """It adapts an event to its calendar."""
    event = EventFactory(address_book)
    assert address_book.calendar == ICalendar(event)



# -*- coding: utf-8 -*-
from datetime import timedelta, date
from icemac.ab.calendar.browser.calendar import EventDescription, hyphenated
from icemac.ab.calendar.browser.calendar import TabularCalendar
from icemac.ab.calendar.browser.interfaces import IEventDescription
from icemac.ab.calendar.browser.interfaces import UnknownLanguageError
from icemac.ab.calendar.event import Event
from icemac.ab.calendar.interfaces import ICalendarDisplaySettings, ICalendar
from icemac.ab.calendar.interfaces import IEvent, IRecurringEvent
from icemac.ab.calendar.testing import get_recurred_event
from icemac.addressbook.interfaces import IEntity
from mock import Mock, patch
from zope.interface.verify import verifyObject
from zope.preference.interfaces import IDefaultPreferenceProvider
from zope.security.interfaces import Unauthorized
from zope.testbrowser.browser import LinkNotFoundError
import pytest
import pytz
import zope.component


MONTH_FOR_TEST = 'November' if date.today().month == 5 else 'May'

# Helper functions


def set_event_additional_fields(address_book, *field_names):
    """Set the given fields to be displayed when an event is rendered."""
    event_entity = IEntity(IEvent)
    ICalendarDisplaySettings(address_book.calendar).event_additional_fields = [
        event_entity.getRawField(x)
        for x in field_names]


# Tests

def test_calendar__TabularCalendar__1(
        address_book, browser, EventFactory, DateTime):
    """It allows a visitor to access a filled calendar."""
    EventFactory(
        address_book, datetime=DateTime.now, alternative_title=u"Qwrtz")
    browser.login('cal-visitor')
    # We need to set a language as otherwise there will only be numbers
    # instead of week day names in the calendar:
    browser.lang('en')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert browser.CALENDAR_OVERVIEW_URL == browser.getLink('Calendar').url
    browser.getLink('Calendar').click()
    assert browser.CALENDAR_MONTH_OVERVIEW_URL == browser.url
    assert 'Sunday' in browser.contents
    assert 'Qwrtz' in browser.contents


def test_calendar__TabularCalendar__2(address_book, browser):
    """It allows a visitor to change his the time zone settings."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    browser.getLink('UTC').click()
    assert browser.PREFS_TIMEZONE_URL.startswith(browser.url)
    browser.getControl('Time zone').displayValue = ['Indian/Christmas']
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message


def test_calendar__TabularCalendar__3(address_book, browser):
    """It does not render an event add link for a visitor."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    # No link to add events:
    with pytest.raises(LinkNotFoundError):
        browser.getLink('event')


def test_calendar__TabularCalendar__4(address_book, browser):
    """It prevents access of anonymous."""
    browser.handleErrors = False  # needed to catch exception
    with pytest.raises(Unauthorized):
        browser.open(browser.CALENDAR_OVERVIEW_URL)


def test_calendar__TabularCalendar__selected_css_class__1(zcmlS):
    """It returns 'month' as selected if there is nothing in the session."""
    view = TabularCalendar()
    view.request = Mock()
    with patch('icemac.addressbook.browser.base.get_session') as session:
        session().get.return_value = None
        assert 'selected' == view.selected_css_class('month')
        assert None is view.selected_css_class('year')


def test_calendar__TabularCalendar__selected_css_class__2(zcmlS):
    """It returns the view stored in the session as selected."""
    view = TabularCalendar()
    view.request = Mock()
    with patch('icemac.addressbook.browser.base.get_session') as session:
        session().get.return_value = 'year'
        assert 'selected' == view.selected_css_class('year')
        assert None is view.selected_css_class('month')


def test_calendar__MonthSelectorForm__1(address_book, browser, DateTime):
    """It displays the current month by default."""
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the
    # month name is not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    current_month = DateTime.now.strftime('%B')
    current_year = str(DateTime.now.year)
    assert [current_month] == browser.getControl('month').displayValue
    assert [current_year] == browser.getControl('year').displayValue


def test_calendar__MonthSelectorForm__2(address_book, browser):
    """It can switch to the entered month."""
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the
    # month name is not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getControl('month').getControl(MONTH_FOR_TEST).click()
    browser.getControl('year').getControl('2024').click()
    browser.getControl('Apply').click()
    assert 'Month changed.' == browser.message
    assert [MONTH_FOR_TEST] == browser.getControl('month').displayValue
    assert ['2024'] == browser.getControl('year').displayValue


def test_calendar__MonthSelectorForm__3(address_book, browser):
    """It keeps the month the user switched to."""
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the
    # month name is not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getControl('month').getControl(MONTH_FOR_TEST).click()
    browser.getControl('year').getControl('2024').click()
    browser.getControl('Apply').click()
    assert 'Month changed.' == browser.message
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert [MONTH_FOR_TEST] == browser.getControl('month').displayValue
    assert ['2024'] == browser.getControl('year').displayValue


def test_calendar__MonthSelectorForm__4(address_book, browser):
    """It translates the months in the month dropdown."""
    browser.login('cal-visitor')
    browser.lang('de-DE')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert 'Mai' == browser.getControl('month').displayOptions[4]


def test_calendar__Dispatcher__1(address_book, browser):
    """It keeps the view mode switched to."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getLink('Year').click()
    assert browser.CALENDAR_YEAR_OVERVIEW_URL == browser.url
    browser.open(browser.CALENDAR_OVERVIEW_URL)
    assert browser.CALENDAR_YEAR_OVERVIEW_URL == browser.url


def test_calendar__MonthCalendar__1(
        address_book, EventFactory, DateTime, browser):
    """It shows the events belonging to the selected month."""
    now = DateTime.now
    EventFactory(address_book, alternative_title=u'foo bär', datetime=now)
    EventFactory(address_book, alternative_title=u'baz qux',
                 datetime=now + timedelta(days=31))
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert 'foo bär' in browser.contents
    assert 'baz qux' not in browser.contents


def test_calendar__MonthCalendar__2(
        address_book, RecurringEventFactory, CategoryFactory, DateTime,
        browser):
    """It shows recurred events as links to customize them."""
    RecurringEventFactory(
        address_book, alternative_title=u'foo bär', datetime=DateTime.now,
        period=u'weekly', category=CategoryFactory(address_book, u'bat'))
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert browser.getLink(u'foo bär').url.startswith(
        browser.RECURRED_EVENT_CUSTOMIZE_URL)
    assert browser.getLink(u'foo bär').url.endswith(
        '@@customize-recurred-event?date={}&event=RecurringEvent'.format(
            date.today().isoformat()))


def test_calendar__MonthCalendar__3(address_book, browser):
    """It renders the time zone the user has set in the prefs as a link."""
    default_prefs = zope.component.getUtility(IDefaultPreferenceProvider)
    default_prefs.getDefaultPreferenceGroup(
        'ab.timeZone').time_zone = 'Pacific/Fiji'
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert browser.PREFS_TIMEZONE_URL.startswith(
        browser.getLink('Pacific/Fiji').url)


def test_calendar__MonthCalendar__4(
        address_book, EventFactory, DateTime, browser):
    """It shows events in the time zone selected by the user."""
    default_prefs = zope.component.getUtility(IDefaultPreferenceProvider)
    default_prefs.getDefaultPreferenceGroup(
        'ab.timeZone').time_zone = 'America/Los_Angeles'
    EventFactory(address_book, alternative_title=u'1st of april utc',
                 datetime=DateTime(2014, 4, 1, 0))
    EventFactory(address_book, alternative_title=u'2nd of april utc',
                 datetime=DateTime(2014, 4, 2, 0))
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the month
    # name is not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getControl('month').getControl('April').selected = True
    browser.getControl('year').getControl('2014').selected = True
    browser.getControl('Apply').click()
    assert 'Month changed.' == browser.message
    # 1st of april 0:00 UTC is in march for Los Angeles timezone, so it
    # does not show up here.
    assert '1st of april utc' not in browser.contents
    assert '2nd of april utc' in browser.contents


def test_calendar__MonthCalendar__5(address_book, browser):
    """It can switch to the year view."""
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the month
    # names are not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    browser.getLink('Year').click()
    assert browser.CALENDAR_YEAR_OVERVIEW_URL == browser.url
    year = date.today().year
    assert 'January {}'.format(year) in browser.contents
    assert 'December {}'.format(year) in browser.contents


def test_calendar__MonthCalendar__6(
        address_book, FieldFactory, CategoryFactory, DateTime,
        RecurringEventFactory, browser):
    """It displays the fields of event objects selected in master data."""
    # Create user fields for select
    event_field_name = FieldFactory(
        address_book, IEvent, u'Int', u'reservations').__name__
    revent_field_name = FieldFactory(
        address_book, IRecurringEvent, u'Int', u'reservations').__name__
    # Sort order is used for display in calendar:
    set_event_additional_fields(
        address_book, 'text', event_field_name, 'persons')

    category = CategoryFactory(address_book, u'bar')
    data = {'datetime': DateTime.now, 'text': u'Text1',
            'period': 'yearly', revent_field_name: 42,
            'external_persons': [u'Ben Utzer'], 'category': category}
    RecurringEventFactory(address_book, **data)
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MONTH_OVERVIEW_URL)
    assert [
        'Text1',
        '42',
        'Ben Utzer'
    ] == browser.etree.xpath('//ul[@class="info"]/li/text()')


@pytest.mark.webdriver
def test_calendar__MonthCalendar__7_webdriver(address_book, webdriver):
    """It renders previous and next links switching months."""
    cal = webdriver.calendar
    webdriver.login('cal-visitor', cal.CALENDAR_MONTH_OVERVIEW_URL)
    cal.year = 2026
    cal.month = 'January'
    cal.switch_to_previous_month()
    assert 'December' == cal.month
    assert 2025 == cal.year
    cal.switch_to_next_month()
    cal.switch_to_next_month()
    assert 'February' == cal.month
    assert 2026 == cal.year


def test_calendar__YearCalendar__1(
        address_book, EventFactory, DateTime, browser):
    """It shows the events belonging to the selected year."""
    now = DateTime.now
    EventFactory(address_book, alternative_title=u'this year', datetime=now)
    EventFactory(address_book, alternative_title=u'next year',
                 datetime=now + timedelta(days=366))
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_YEAR_OVERVIEW_URL)
    assert 'this year' in browser.contents
    assert 'next year' not in browser.contents


def test_calendar__YearCalendar__2(
        address_book, RecurringEventFactory, TimeZonePrefFactory, DateTime,
        CategoryFactory, browser):
    """It renders events with a constant time even through DST changes."""
    TimeZonePrefFactory('Europe/Berlin')
    RecurringEventFactory(
        address_book,
        datetime=DateTime(
            2014, 3, 11, 8, tzinfo=pytz.timezone('Europe/Berlin')),
        end=DateTime(2014, 5, 2).date(),
        category=CategoryFactory(address_book, u'my cat'),
        period='nth weekday of month')
    browser.login('cal-visitor')
    # We need to explicitly set the language here because otherwise the month
    # name is not displayed:
    browser.lang('en')
    browser.open(browser.CALENDAR_YEAR_OVERVIEW_URL)
    browser.getControl('year').getControl('2014').selected = True
    browser.handleErrors = False
    browser.getControl('Apply').click()
    assert 'Year changed.' == browser.message
    # The time stays the same even there is a DST switch in between:
    assert ['8:00 AM', '8:00 AM'] == browser.etree.xpath('//table//dt/text()')


@pytest.mark.webdriver
def test_calendar_js__1_webdriver(address_book, webdriver):
    """It auto-submits on change in the month drop-down of the month view."""
    calendar = webdriver.calendar
    webdriver.login('cal-visitor', calendar.CALENDAR_MONTH_OVERVIEW_URL)
    calendar.month = MONTH_FOR_TEST
    assert u'Month changed.' == webdriver.message


@pytest.mark.webdriver
def test_calendar_js__2_webdriver(address_book, webdriver):
    """It auto-submits on change in the year drop-down of the month view."""
    calendar = webdriver.calendar
    webdriver.login('cal-visitor', calendar.CALENDAR_MONTH_OVERVIEW_URL)
    calendar.year = 2024
    assert u'Month changed.' == webdriver.message


@pytest.mark.webdriver
def test_calendar_js__3_webdriver(address_book, webdriver):
    """It auto-submits on change in the year drop-down of the year view."""
    calendar = webdriver.calendar
    webdriver.login('cal-visitor', calendar.CALENDAR_YEAR_OVERVIEW_URL)
    calendar.year = 2024
    assert u'Year changed.' == webdriver.message


def test_calendar__EventDescription__1(EventDescriptionFactory):
    """It fulfills the `IEventDescription` interface."""
    event_description = EventDescriptionFactory(Event())
    assert isinstance(event_description, EventDescription)
    assert verifyObject(IEventDescription, event_description)


def test_calendar__EventDescription__persons__1(
        address_book, PersonFactory, EventFactory, EventDescriptionFactory):
    """It is a comma separated list of persons.

    It combines the persons from the address book and the external persons.
    """
    person = PersonFactory(address_book, u'Tester', first_name=u'Hans')
    event = EventFactory(
        address_book, persons=set([person]), external_persons=[u'Heiner Myer'])
    event_description = EventDescriptionFactory(event=event)
    assert u'Hans Tester, Heiner Myer' == event_description.persons


def test_calendar__EventDescription__persons__2(EventDescriptionFactory):
    """It is an empty string if there are no persons assigned to the event."""
    assert u'' == EventDescriptionFactory(Event()).persons


@pytest.fixture('function')
def selected_user_field(address_book, FieldFactory):
    """Create a user defined field ad select it for display.

    Return the name of the user defined field.
    """
    field_name = FieldFactory(
        address_book, IEvent, u'Int', u'reservations').__name__
    set_event_additional_fields(address_book, 'text', field_name)
    return field_name


def test_calendar__EventDescription__getInfo__1(
        address_book, CategoryFactory, RecurringEventFactory, DateTime,
        selected_user_field, utc_time_zone_pref):
    """It ignores not existing fields.

    This happens if a user defined field is chosen for display but there is no
    equivalent field on the recurring event which is rendered as recurred
    event.

    """
    recurring_event = RecurringEventFactory(address_book, **{
        'datetime': DateTime.now,
        'text': u'Text2',
        'period': 'daily',
        'category': CategoryFactory(address_book, u'bar')})
    ed = IEventDescription(get_recurred_event(recurring_event, DateTime))
    assert ['Text2'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__2(
        address_book, EventFactory, DateTime, CategoryFactory,
        selected_user_field, utc_time_zone_pref):
    """It ignores a selected but later deleted user defined field."""
    ed = IEventDescription(EventFactory(address_book, **{
        'datetime': DateTime.now,
        'text': u'Text1',
        'category': CategoryFactory(address_book, u'bar'),
        selected_user_field: 42}))
    event_entity = IEntity(IEvent)
    event_entity.removeField(event_entity.getRawField(selected_user_field))
    assert ['Text1'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__3(
        address_book, FieldFactory, EventFactory, utc_time_zone_pref):
    """It returns a list of the selected fields as unicode objects."""
    reservations = FieldFactory(
        address_book, IEvent, u'Int', u'reservations').__name__
    # Both user fields and pre-defined fields are possible
    set_event_additional_fields(address_book, 'text', reservations)
    ed = IEventDescription(EventFactory(address_book, **{
        'text': u'Event is not yet sure.',
        reservations: 50}))
    assert [u'Event is not yet sure.', u'50'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__4(
        address_book, EventFactory, utc_time_zone_pref):
    """It splits the text field at line endings."""
    set_event_additional_fields(address_book, 'text')
    ed = IEventDescription(EventFactory(address_book, text=u'foo\nbar'))
    assert [u'foo', u'bar'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__5(
        address_book, EventFactory, utc_time_zone_pref):
    """It omits fields with a `None` value."""
    set_event_additional_fields(address_book, 'text')
    ed = IEventDescription(EventFactory(address_book))
    assert ed.context.text is None
    assert [] == ed.getInfo()


def test_calendar__EventDescription__getInfo__6(
        address_book, EventFactory, utc_time_zone_pref):
    """It omits empty string values."""
    set_event_additional_fields(address_book, 'persons')
    ed = IEventDescription(EventFactory(address_book))
    assert '' == ed.persons
    assert [] == ed.getInfo()


def test_calendar__EventDescription__getInfo__7(
        address_book, FieldFactory, EventFactory, utc_time_zone_pref):
    """It does not omit numbers with the value of zero."""
    num_field = FieldFactory(address_book, IEvent, u'Int', u'num').__name__
    set_event_additional_fields(address_book, num_field)
    ed = IEventDescription(EventFactory(address_book, **{num_field: 0}))
    assert [u'0'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__8(
        address_book, PersonFactory, EventFactory, utc_time_zone_pref):
    """It returns external and internal persons if `persons` is  selected."""
    p1 = PersonFactory(address_book, u'Tester', first_name=u'Hans')
    set_event_additional_fields(address_book, 'persons')
    ed = IEventDescription(EventFactory(
        address_book,
        persons=set([p1]),
        external_persons=[u'Franz Vrozzek', u'Fritz Vrba']))
    assert [u'Franz Vrozzek, Fritz Vrba, Hans Tester'] == ed.getInfo()


def test_calendar__EventDescription__getInfo__9(
        address_book, EventFactory, utc_time_zone_pref):
    """It hyphenates the text."""
    set_event_additional_fields(address_book, 'text')
    ed = IEventDescription(EventFactory(
        address_book, text=u'I contain longer words.'))
    assert [u'I con&shy;tain longer word&shy;s.'] == ed.getInfo(lang='en')


def test_calendar__EventDescription__getText__1(EventDescriptionFactory):
    """It returns the alternative title if it is set."""
    ed = EventDescriptionFactory(
        Event(), category_name=u'birthday', alternative_title=u'foo bar')
    assert u'foo bar' == ed.getText()


def test_calendar__EventDescription__getText__2(
        address_book, EventDescriptionFactory, CategoryFactory):
    """It returns the category title if the alternative title is not set."""
    ed = EventDescriptionFactory(
        Event(), category=CategoryFactory(address_book, u'foo'),
        alternative_title=None)
    assert u'foo' == ed.getText()


def test_calendar__EventDescription__getText__3(EventDescriptionFactory):
    """It returns 'event' if neither alternative title nor category is set."""
    ed = EventDescriptionFactory(
        Event(), category=None, alternative_title=None)
    assert u'event' == ed.getText()


def test_calendar__EventDescription__getText__4(EventDescriptionFactory):
    """It returns not hyphenated text by default."""
    ed = EventDescriptionFactory(Event(), alternative_title=u'birthday')
    assert u'birthday' == ed.getText()


def test_calendar__EventDescription__getText__5(EventDescriptionFactory):
    """It raises `UnknownLanguageError` for an unknown language."""
    ed = EventDescriptionFactory(Event())
    with pytest.raises(UnknownLanguageError):
        ed.getText(lang='Clingon')


def test_calendar__EventDescription__getText__6(EventDescriptionFactory):
    """getText_returns_hyphenated_respecting_set_language."""
    ed = EventDescriptionFactory(Event(), alternative_title=u'Geburtstag')
    assert u'Ge&shy;burts&shy;tag' == ed.getText(lang='de')


def test_calendar__hyphenated__1():
    """It quotes text as html even if not hyphenating when no lang is set."""
    @hyphenated
    def func(ignored):
        return u'<script>'
    assert u'&lt;script&gt;' == func(any)


def test_calendar__hyphenated__2():
    """It hyphenates text and encodes it text for html."""
    @hyphenated
    def func(ignored, lang=None):
        return u'Gebürtstag<>'

    res = func(any, lang='de')
    assert isinstance(res, unicode)
    assert u'Ge&shy;bürts&shy;tag&lt;&gt;' == res


def test_calendar__calendar_for_event_description__1(
        address_book, EventFactory, EventDescriptionFactory):
    """It adapts IEventDescription to ICalendar."""
    ed = EventDescriptionFactory(EventFactory(address_book))
    assert address_book.calendar == ICalendar(ed)


def test_calendar__event_for_event_description__1(
        address_book, EventFactory, EventDescriptionFactory):
    """It adapts IEventDescription to IEvent."""
    event = EventFactory(address_book)
    ed = EventDescriptionFactory(event)
    assert event == IEvent(ed)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .base import Calendar
from ..interfaces import UnknownLanguageError
from icemac.addressbook.browser.base import can_access_uri_part
from icemac.addressbook.i18n import _
import datetime
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces
import zope.component


SPECIAL_CLASS_MAPPING = {
    True: 'red',
}


def render_event_time(datetime, whole_day, request, no_time=''):
    """Render time of event human readable and localized to locale of user."""
    if whole_day:
        return no_time
    formatter = request.locale.dates.getFormatter(
        'time', 'short')
    time = formatter.format(datetime)
    if request.locale.id.language == 'de':
        time += ' Uhr'
    return time


class TableEvent(icemac.addressbook.browser.base.BaseView):
    """View to render an event in the table."""

    action_class = 'edit'
    _action_url = None
    default_text = _('Edit')  # allow at least to edit the entry

    def time(self):
        zero_width_space = '&#x200b;'  # keep printing uniform
        return render_event_time(
            self.context.datetime, self.context.whole_day, self.request,
            no_time=zero_width_space)

    def dd_class(self):
        return SPECIAL_CLASS_MAPPING.get(self.context.special_event)

    def dt_class(self):
        if self.context.whole_day:
            return 'no-screen'

    def _localized(self, func):
        locale = self.request.locale
        try:
            result = func(locale.getLocaleID())
        except UnknownLanguageError:
            # No hypenation dict for the locale id (e. g. de_DE):
            try:
                # Try only the language name:
                result = func(locale.id.language)
            except UnknownLanguageError:
                # Disable hypenation for unknown languages:
                result = func()
        return result

    def text(self):
        text = self._localized(self.context.getText).strip()
        if not text:
            text = self.default_text
        return text

    def info(self):
        return [{'info': x} for x in self._localized(self.context.getInfo)]

    def action_url(self):
        return self.url(self.context.context)


def get_day_names(request):
    """Return a list of localized week day names starting with Sunday."""
    locale_calendar = request.locale.dates.calendars['gregorian']
    day_names = locale_calendar.getDayNames()
    # Move Sunday to position one
    day_names.insert(0, day_names.pop(-1))
    return day_names


class Table(Calendar):
    """Tabular display of a calendar."""

    grok.name('table')
    render_event_adapter_name = 'table-event'

    def update(self):
        first_of_month = self.month.firstOfMonth()
        if first_of_month.isoweekday() == 7:
            self.first_table_day = first_of_month
        else:
            self.first_table_day = (
                first_of_month - datetime.timedelta(
                    first_of_month.isoweekday()))
        end_of_month = self.month.lastOfMonth()
        self.num_of_days = (end_of_month - self.first_table_day).days + 1
        if end_of_month.isoweekday() == 7:
            self.num_of_days += 6
        else:
            self.num_of_days += 6 - end_of_month.isoweekday()

    def table_head(self):
        self.write('<table>')
        self.write('  <thead>')
        self.write('    <tr>')
        for index, day_name in enumerate(get_day_names(self.request)):
            self.write('      <th class="day-%s">%s</th>', index, day_name)
        self.write('    </tr>')
        self.write('  </thead>')

    def render(self):
        calendar = self.request.locale.dates.calendars['gregorian']
        day_names = calendar.getDayAbbreviations()
        # Move Sunday to position one
        day_names.insert(0, day_names.pop(-1))
        self.table_head()
        self.write('  <tbody>')
        today = datetime.date.today()
        events = self.events
        add_event_for_day_url = self.get_add_event_for_day_url()
        for delta in xrange(self.num_of_days):
            if delta % 7 == 0:
                # week start
                self.write('    <tr>')
            day = self.first_table_day + datetime.timedelta(delta)
            day_number = int(day.strftime('%w'))
            css_classes = ['day-{}'.format(day_number)]
            if day == today:
                css_classes.append('today')
            self.write('<td class="%s">', ' '.join(css_classes), newline=False)
            if day in self.month:
                self.write('<span class="weekday no-screen">%s</span>',
                           day_names[day_number])
                if add_event_for_day_url:
                    self.write(
                        '<a class="wrapper" href="%s?date=%s" title="%s">',
                        add_event_for_day_url, day.isoformat(),
                        self.translate(_('Add new event for this day.')))
                else:
                    self.write('<span class="wrapper">')
                self.write('<span class="number">%s</span>', day.day)
                if add_event_for_day_url:
                    self.write('</a>')
                else:
                    self.write('</span>')
                found_events_for_day = False
                for ev in events[:]:
                    if ev.datetime.date() != day:
                        # events are sorted, so we can break on day in future
                        break
                    # remove day from list to print to keep for-loop short
                    events.pop(events.index(ev))
                    if not found_events_for_day:
                        self.write('<dl>')
                        found_events_for_day = True
                    view = zope.component.getMultiAdapter(
                        (ev, self.request),
                        name=self.render_event_adapter_name)
                    self.write(view())
                if found_events_for_day:
                    self.write('</dl>')
            self.write('</td>')
            if (delta + 1) % 7 == 0:
                # week end
                self.write('    </tr>')

        self.write('  </tbody>')
        self.write('</table>')
        return self.read()

    def get_add_event_for_day_url(self):
        """Get the URL to add a new a event for a day.

        Returns None if there should be no add link.
        """
        calendar = icemac.ab.calendar.interfaces.ICalendar(
            icemac.addressbook.interfaces.IAddressBook(None))
        if not can_access_uri_part(calendar, self.request, 'addEvent.html'):
            return None
        return self.url(calendar, 'addEvent.html')



import pytest


@pytest.mark.parametrize('username,url', [
    ('cal-visitor', 'CALENDAR_MONTH_OVERVIEW_URL'),
    ('cal-editor', 'EVENT_ADD_URL')])
def test_resource__1(address_book, browser, username, url):
    """The `calendar.css` is rendered on calendar pages."""
    browser.login(username)
    browser.open(getattr(browser, url))
    assert 'href="/fanstatic/calendar/:version:' in browser.contents
    assert 'calendar.css' in browser.contents



from icemac.addressbook.i18n import _
import gocept.reference.interfaces
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.masterdata.breadcrumb
import icemac.addressbook.browser.base
import icemac.addressbook.browser.breadcrumb
import icemac.addressbook.browser.metadata
import icemac.addressbook.browser.table
import z3c.table.column


class CategoriesBreadCrumb(
        icemac.ab.calendar.masterdata.breadcrumb.CalendarMDChildBreadcrumb):
    """Breadcrumb for categories."""

    grok.adapts(
        icemac.ab.calendar.interfaces.ICategories,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Event categories')


class Table(icemac.addressbook.browser.table.Table):
    """List event categories."""

    title = icemac.addressbook.browser.breadcrumb.DO_NOT_SHOW
    no_rows_message = _(u'No event categories defined yet.')

    def setUpColumns(self):
        return [z3c.table.column.addColumn(
            self, icemac.addressbook.browser.table.TitleLinkColumn, 'title',
            header=_(u'event category'))]

    @property
    def values(self):
        """The values are stored on the context."""
        return self.context.values()


class Add(icemac.addressbook.browser.base.BaseAddForm):
    """Add form for event category."""

    title = _(u'Add new event category')
    interface = icemac.ab.calendar.interfaces.ICategory
    class_ = icemac.ab.calendar.category.Category
    next_url = 'parent'


def can_delete_category(form):
    """Button condition telling if the displayed category can be deleted."""
    return (
        icemac.addressbook.browser.base.can_access('@@delete.html')(form) and
        not gocept.reference.interfaces.IReferenceTarget(
            form.context).is_referenced())


class Edit(icemac.addressbook.browser.base.GroupEditForm):
    """Edit form for event category."""

    title = _(u'Edit event category')
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    interface = icemac.ab.calendar.interfaces.ICategory
    next_url = 'parent'
    z3c.form.form.extends(icemac.addressbook.browser.base.GroupEditForm,
                          ignoreFields=True)

    def applyChanges(self, data):
        # GroupForm has its own applyChanges but we need the one from
        # _AbstractEditForm here as inside the goups no changes are made but
        # there is a subscriber which raises an error which is handled by
        # _AbstractEditForm.
        return icemac.addressbook.browser.base._AbstractEditForm.applyChanges(
            self, data)

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete', condition=can_delete_category)
    def handleDelete(self, action):
        self.redirect_to_next_url('object', 'delete.html')


class Delete(icemac.addressbook.browser.base.BaseDeleteForm):
    """Form to delete a category."""

    title = _('Delete event category')
    label = _('Do you really want to delete this event category?')
    interface = icemac.ab.calendar.interfaces.ICategory
    field_names = ('title', )



from .. testing import DateTimeClass
import datetime


def test_testing__DateTimeClass__format__1():
    """It formats a datetime to a date if forced to."""
    dt = datetime.datetime(2017, 1, 31, 13, 32)
    assert '17/01/31 13:32' == DateTimeClass().format(dt)
    assert '2017 1 31 ' == DateTimeClass().format(dt, force_date=True)



from icemac.ab.calendar.browser.interfaces import IDatetime
from mock import Mock
import datetime
import pytest
import zope.schema


@pytest.fixture('session')
def DatetimeFactory():
    """Factory to create a mock object implementing the IDatetime interface."""
    def create_datetime(date, time, whole_day_event):
        mock = Mock()
        mock.date = datetime.date(2015, 4, 15)
        if time:
            mock.time = datetime.time(18, 29)
        else:
            mock.time = None
        mock.whole_day_event = bool(whole_day_event)
        return mock
    return create_datetime


def test_interfaces__IDatetime__1(DatetimeFactory):
    """A whole day event with time is valid."""
    dt = DatetimeFactory('date', 'time', 'whole_day')
    assert [] == zope.schema.getValidationErrors(IDatetime, dt)


def test_interfaces__IDatetime__2(DatetimeFactory):
    """A whole day event without time is valid."""
    dt = DatetimeFactory('date', None, 'whole_day')
    assert [] == zope.schema.getValidationErrors(IDatetime, dt)


def test_interfaces__IDatetime__3(DatetimeFactory):
    """A non-whole day event with time is valid."""
    dt = DatetimeFactory('date', 'time', whole_day_event=False)
    assert [] == zope.schema.getValidationErrors(IDatetime, dt)


def test_interfaces__IDatetime__4(DatetimeFactory):
    """A non-whole day event without time is invalid."""
    dt = DatetimeFactory('date', None, whole_day_event=False)
    v = zope.schema.getValidationErrors(IDatetime, dt)
    assert 1 == len(v)
    assert 'Either enter a `time` or select `whole day event`!' == str(v[0][1])



from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.eventview.interfaces
import icemac.ab.calendar.eventview.model
import icemac.ab.calendar.masterdata.breadcrumb
import icemac.addressbook.browser.base
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.metadata
import icemac.addressbook.browser.table
import z3c.form.button
import z3c.form.form
import z3c.table.column


class RecurringEventsBreadCrumb(
        icemac.ab.calendar.masterdata.breadcrumb.CalendarMDChildBreadcrumb):
    """Breadcrumb for event views."""

    grok.adapts(
        icemac.ab.calendar.eventview.interfaces.IEventViewContainer,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Event views')


event_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    icemac.ab.calendar.eventview.interfaces.IEventViewContainer,
    icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration,
)


class Table(icemac.addressbook.browser.table.Table):
    """List event views."""

    title = icemac.addressbook.browser.breadcrumb.DO_NOT_SHOW
    no_rows_message = _('No event views defined yet.')

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'title'),
        ]

    @property
    def values(self):
        return self.context.values()


class Add(icemac.addressbook.browser.base.BaseAddForm):
    """Add form for an event view configuration."""

    title = _(u'Add new event view configuration')
    interface = icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration
    class_ = icemac.ab.calendar.eventview.model.EventViewConfiguration
    next_url = 'parent'


class Edit(icemac.addressbook.browser.base.GroupEditForm):
    """Edit form for event configuration."""

    title = _(u'Edit event view configuration')
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    interface = icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration
    next_url = 'parent'
    z3c.form.form.extends(icemac.addressbook.browser.base.GroupEditForm,
                          ignoreFields=True)

    def applyChanges(self, data):
        # GroupForm has its own applyChanges but we need the one from
        # _AbstractEditForm here as inside the groups no changes are made but
        # there is a subscriber which raises an error which is handled by
        # _AbstractEditForm.
        return icemac.addressbook.browser.base._AbstractEditForm.applyChanges(
            self, data)

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete',
        condition=icemac.addressbook.browser.base.can_access('@@delete.html'))
    def handleDelete(self, action):
        self.redirect_to_next_url('object', 'delete.html')


class Delete(icemac.addressbook.browser.base.BaseDeleteForm):
    """Confirm delete of event view configuration."""

    title = _('Delete event view configuration')
    label = _('Do you really want to delete this event view configuration?')
    interface = icemac.ab.calendar.eventview.interfaces.IEventViewConfiguration



from icemac.ab.calendar.interfaces import ICalendar, ICategories
from icemac.ab.calendar.interfaces import IRecurringEvents
from icemac.ab.calendar.eventview.interfaces import IEventViewContainer
from icemac.addressbook.addressbook import AddressBookCreated
from icemac.ab.calendar.install import install_calendar


def test_install__install_calendar__1(assert_address_book):
    """It creates an calendar attribute."""
    install_calendar(AddressBookCreated(assert_address_book.address_book))
    assert_address_book.has_attribute('calendar', ICalendar)
    assert_address_book.has_attribute('calendar_categories', ICategories)
    assert_address_book.has_attribute(
        'calendar_recurring_events', IRecurringEvents)
    assert_address_book.has_attribute(
        'calendar_eventviews', IEventViewContainer)


def test_install__install_calendar__2(assert_address_book):
    """It does not break if it gets called twice."""
    event = AddressBookCreated(assert_address_book.address_book)
    install_calendar(event)
    install_calendar(event)
    assert_address_book.has_attribute('calendar', ICalendar)



# -*- coding: utf-8 -*-
from .interfaces import IEventDescription
from .interfaces import UnknownLanguageError
from datetime import date
from icemac.addressbook.i18n import _
import cgi
import collections
import copy
import decorator
import gocept.month
import grokcore.component as grok
import icemac.ab.calendar.browser.base
import icemac.ab.calendar.browser.renderer.interfaces
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces
import icemac.addressbook.preferences.utils
import pyphen
import pytz
import z3c.form.field
import z3c.formui.form
import zc.sourcefactory.basic
import zope.cachedescriptors.property
import zope.component
import zope.globalrequest
import zope.interface


class Dispatcher(icemac.ab.calendar.browser.base.View):
    """Dispatch to month resp. year view."""

    possible_views = collections.defaultdict(
        lambda: 'month.html', {
            'year': 'year.html',
            'event-view': 'event-view.html',
        })
    known_targets = tuple(possible_views.keys()) + ('month',)

    def __call__(self):
        target = self.request.get('to', None)
        if target in self.known_targets:
            self.session['calendar_view'] = target
        else:
            target = self.session.get('calendar_view', 'month')
        self.request.response.redirect(
            self.url(self.context, self.possible_views[target]))
        return ''


class MonthSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Enumerates months."""

    def getValues(self):
        return range(1, 13)

    def getTitle(self, value):
        request = zope.globalrequest.getRequest()
        calendar = request.locale.dates.calendars['gregorian']
        return calendar.months.get(value)[0]


month_source = MonthSource()


class YearSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Enumerate 5 years past + 10 years future."""

    def getValues(self):
        current_year = date.today().year
        return range(current_year - 5, current_year + 11)


year_source = YearSource()


class IMonthSelector(zope.interface.Interface):
    """Select a month for display."""

    calendar_month = zope.schema.Choice(title=_('month'), source=month_source)
    calendar_year = zope.schema.Choice(title=_('year'), source=year_source)


class MonthSelectorForm(icemac.addressbook.browser.base.BaseForm,
                        z3c.formui.form.EditForm):
    """Form to enter the month which should be displayed in the calendar."""

    fields = z3c.form.field.Fields(IMonthSelector)
    successMessage = _('Month changed.')
    id = 'calendar-select-form'


class TabularCalendar(icemac.ab.calendar.browser.base.View):
    """Tabular calendar display."""

    form_class = NotImplemented
    css_class = NotImplemented
    month_names = None  # set in `update()`

    @property
    def calendar_year(self):
        year = self.session.get('calendar_year')
        if year is None:
            # Store default value:
            self.calendar_year = year = gocept.month.Month.current().year
        return year

    @calendar_year.setter
    def calendar_year(self, value):
        self.session['calendar_year'] = value

    def selected_css_class(self, name):
        calendar_view = self.session.get('calendar_view')
        if calendar_view == name:
            return 'selected'
        # no selection in session, use default:
        if calendar_view is None and name == 'month':
            return 'selected'

    def menu_url(self, target):
        return self.url(self.context, to=target)

    def update(self):
        locale_calendar = self.request.locale.dates.calendars['gregorian']
        self.month_names = locale_calendar.getMonthNames()
        self.update_form()

    def update_form(self):
        self.form = self.form_class(self, self.request)
        # Write the value the user entered on self:
        self.form.update()

    def render_form(self):
        return self.form.render()

    def time_zone_name(self):
        """User selected time zone name."""
        return icemac.addressbook.preferences.utils.get_time_zone_name()

    def time_zone_prefs_url(self):
        return self.url(icemac.addressbook.interfaces.IAddressBook(self),
                        '++preferences++/ab.timeZone')

    def get_month_name(self, month):
        """Render the month name of a gocept.month instance."""
        return self.month_names[month.month - 1]

    def previous_link(self):
        return ''

    def next_link(self):
        return ''

    def events_in_interval(self, start, end, condition=lambda x: True):
        """Get all events in an interval filtered by a condition function.

        Returns list of tuples (month, [event-description, event-description]).
        """
        year = gocept.month.MonthInterval(start, end)
        return [(
            month,
            [IEventDescription(x)
             for x in self.context.get_events_for_month(
                month, self.time_zone_name())
             if condition(x)]
        ) for month in year]


@zope.interface.implementer(IMonthSelector)
class MonthCalendar(TabularCalendar):
    """Calendar display of one month."""

    form_class = MonthSelectorForm
    css_class = 'month'
    renderer_name = 'table'
    title = _('Month tabular view')

    @property
    def calendar_month(self):
        month = self.session.get('calendar_month')
        if month is None:
            # Store default value:
            self.calendar_month = month = gocept.month.Month.current().month
        return month

    @calendar_month.setter
    def calendar_month(self, value):
        self.session['calendar_month'] = value

    @zope.cachedescriptors.property.Lazy
    def month(self):
        """Month which should get displayed."""
        return gocept.month.Month(self.calendar_month, self.calendar_year)

    def update(self):
        super(MonthCalendar, self).update()
        self.renderer = zope.component.getMultiAdapter(
            (self.month, self.request, self.get_event_descriptions()),
            icemac.ab.calendar.browser.renderer.interfaces.IRenderer,
            name=self.renderer_name)

    def previous_link(self):
        prev_month = self.month - 1
        url = '#month={0.month}&year={0.year}'.format(prev_month)
        return {'url': url,
                'month': prev_month.month,
                'year': prev_month.year,
                'text': u'◄'}

    def next_link(self):
        next_month = self.month + 1
        url = '#month={0.month}&year={0.year}'.format(next_month)
        return {'url': url,
                'month': next_month.month,
                'year': next_month.year,
                'text': u'►'}

    def get_event_descriptions(self):
        return [IEventDescription(x)
                for x in self.context.get_events_for_month(
                    self.month, self.time_zone_name())]

    def render_calendar(self):
        return u'<h2 class="no-screen">{month} {year}</h2>\n{cal}'.format(
            month=self.get_month_name(self.month), year=self.month.year,
            cal=self.renderer())


class IYearSelector(zope.interface.Interface):
    """Select a month for display."""

    calendar_year = zope.schema.Choice(title=_('year'), source=year_source)


class YearSelectorForm(icemac.addressbook.browser.base.BaseForm,
                       z3c.formui.form.EditForm):
    """Form to enter the year which should be displayed in the calendar."""

    fields = z3c.form.field.Fields(IYearSelector)
    successMessage = _('Year changed.')
    id = 'calendar-select-form'


@zope.interface.implementer(IYearSelector)
class YearCalendar(TabularCalendar):
    """Calendar displaying a whole year."""

    form_class = YearSelectorForm
    css_class = 'year'
    title = _('Year tabular view')

    def update(self):
        super(YearCalendar, self).update()
        self.events = self.events_in_interval(
            gocept.month.Month(1, self.calendar_year),
            gocept.month.Month(12, self.calendar_year))

    def render_events(self, month, events):
        prefix = u'<div class="month">\n<h2>{} {}</h2>'.format(
            self.get_month_name(month), month.year)
        suffix = u'</div>'
        calendar = zope.component.getMultiAdapter(
            (month, self.request, events),
            icemac.ab.calendar.browser.renderer.interfaces.IRenderer,
            name='table')()
        return '\n'.join((prefix, calendar, suffix))

    def render_calendar(self):
        result = [self.render_events(month, events)
                  for month, events in self.events]
        return '\n'.join(result)


SOFT_HYPHEN = u'\u00AD'
SOFT_HYPHEN_HTML = u'&shy;'


def hyphenate(text, dic):
    """Hyphenate a `text` using `dic`tionary for display in browser."""
    # HTML escape needs to be done *after* the hyphenation, otherwise the
    # entities are hyphenated. Thus there is a two step process:
    # 1. Hyphenate with a special marker
    # 2. replace marker with the actual entity
    if dic is None:
        hyphenated = text
    else:
        words = text.split(u' ')
        hyphenated = u' '.join(
            [dic.inserted(word, SOFT_HYPHEN) for word in words])
    escaped = cgi.escape(hyphenated)
    return escaped.replace(SOFT_HYPHEN, SOFT_HYPHEN_HTML)


@decorator.decorator
def hyphenated(func, context, lang=None):
    """Decorator for methods those return value should be hyphenated.

    Usage:

    @hyphenated
    def method(self, lang=None):
        return text

    Call it using: self.method(lang='de')

    """
    if lang is None:
        dic = None
    else:
        try:
            dic = pyphen.Pyphen(lang=lang)
        except KeyError:
            # Fail early if we cannot hyphen the desired language:
            raise UnknownLanguageError()
    text = func(context)
    if isinstance(text, list):
        text = [hyphenate(x, dic) for x in text]
    else:
        text = hyphenate(text, dic)
    return text


class EventDescriptionBase(object):
    """Base class for event adapters."""

    def __init__(self, context):
        self.context = context
        timezone = pytz.timezone(
            icemac.addressbook.preferences.utils.get_time_zone_name())
        if context.datetime is None:
            self.datetime = None
        else:
            self.datetime = context.in_timezone(timezone)
        self.whole_day = context.whole_day_event
        self._text = icemac.addressbook.interfaces.ITitle(context)
        self.persons = u', '.join(context.listPersons())

    def _get_info_from_fields(self, fields):
        """Convert a list of entitiy fields to a list of data."""
        info = []
        for field in fields:
            if field is icemac.ab.calendar.interfaces.IEvent['persons']:
                value = self.persons
            else:
                schema_field = (
                    icemac.addressbook.entities.get_bound_schema_field(
                        self.context, None, field,
                        default_attrib_fallback=False))
                try:
                    value = schema_field.get(schema_field.context)
                except AttributeError:
                    # Field defined on IEvent but not on IRecurringEvent, thus
                    # it does not exist on the RecurredEvent.
                    value = None
                if value is not None:
                    value = unicode(value)
            if value:
                if field is icemac.ab.calendar.interfaces.IEvent['text']:
                    info.extend(value.split('\n'))
                else:
                    info.append(value)
        return info


class EventDescription(EventDescriptionBase, grok.Adapter):
    """Adapter from IEvent to IEventDescription needed by renderer."""

    grok.context(icemac.ab.calendar.interfaces.IBaseEvent)
    grok.implements(IEventDescription)

    def __init__(self, context):
        super(EventDescription, self).__init__(context)
        self.prio = 0
        self.whole_day = context.whole_day_event
        self.special_event = None
        calendar = icemac.ab.calendar.interfaces.ICalendar(context)
        # Without the following line we get a ForbidenError for
        # `__getitem__` when accessing the annotations where
        # `ICalendarDisplaySettings` are stored. As only authorized users
        # are able to access this adapter, this is no security hole.
        unsave_calendar = zope.security.proxy.getObject(calendar)
        # Making a copy so changing is not possible:
        self._info_fields = copy.copy(
            icemac.ab.calendar.interfaces.ICalendarDisplaySettings(
                unsave_calendar).event_additional_fields)

    @hyphenated
    def getText(self, lang=None):
        return self._text

    @hyphenated
    def getInfo(self, lang=None):
        return self._get_info_from_fields(self._info_fields)


@grok.adapter(IEventDescription)
@grok.implementer(icemac.ab.calendar.interfaces.IEvent)
def event_for_event_description(ed):
    """Get the event an event description was created from."""
    return ed.context


@grok.adapter(IEventDescription)
@grok.implementer(icemac.ab.calendar.interfaces.ICalendar)
def calendar_for_event_description(ed):
    """Get the calendar an event description belongs to."""
    return icemac.ab.calendar.interfaces.ICalendar(ed.context)



from icemac.addressbook.i18n import _
import z3c.layer.pagelet
import zope.interface
import zope.schema


class ICalendarLayer(z3c.layer.pagelet.IPageletBrowserLayer):
    """Calendar browser layer."""


class IDatetime(zope.interface.Interface):
    """Object interface to edit datetime data."""

    whole_day_event = zope.schema.Bool(
        title=_('whole day event?'), default=False)
    date = zope.schema.Date(title=_('date'), required=True)
    time = zope.schema.Time(title=_('time'), required=False)
    datetime = zope.interface.Attribute(
        '`date` and `time` combined to a datetime.')

    @zope.interface.invariant
    def on_non_whole_day_event_time_must_be_set(event):
        if not event.whole_day_event and event.time is None:
            raise zope.interface.Invalid(
                _('Either enter a `time` or select `whole day event`!'))


class IEventDatetime(zope.interface.Interface):
    """Interface to edit event's date and time."""

    datetime = zope.schema.Object(title=_('datetime'), schema=IDatetime)


class IEventDescription(zope.interface.Interface):
    """Description of a single event which can be rendered in the calender."""

    context = zope.interface.Attribute('IEvent this description is based on.')

    datetime = zope.interface.Attribute('datetime.datetime object')
    prio = zope.interface.Attribute(
        'Event descriptions for the same `datetime` and `kind` with a higher '
        '`prio` override the ones with lower `prio`.')

    whole_day = zope.interface.Attribute(
        'Event is the whole day, so do not display time.')

    persons = zope.interface.Attribute(
        'Comma separated list of person names belonging to the event.')

    def getText(lang=None):
        """Textual description of the event.

        If `lang` is not `None` a hyphenation dict for this language is
        looked up. This might raise a `LookupError`. Otherwise the text is
        hyphenated for HTML.
        """

    def getInfo(lang=None):
        """List of additional information about the event.

        The contents of the list are defined in master data of calendar.

        """


class UnknownLanguageError(LookupError):
    """Error indicating an unknown laguage."""



# -*- coding: utf-8 -*-
from .interfaces import DATE_INDEX
from .interfaces import IEvent
from .interfaces import IRecurringEvent
from datetime import datetime, time
from icemac.addressbook.interfaces import ISchemaName
import gocept.reference
import grokcore.annotation as grok
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.property
import icemac.addressbook.interfaces
import itertools
import pytz
import zope.catalog.interfaces
import zope.component
import zope.container.btree
import zope.interface


@zope.interface.implementer(icemac.ab.calendar.interfaces.ICalendar)
class Calendar(zope.container.btree.BTreeContainer):
    """Calendar containing dates."""

    def get_events_for_month(self, month, timezone=None):
        """Get all events which belong to `month`."""
        timezone = self._timezone_name_to_timezone(timezone)
        midnight = time(0, 0, 0)
        start = timezone.localize(
            datetime.combine(month.firstOfMonth(), midnight))
        end = timezone.localize(
            datetime.combine((month + 1).firstOfMonth(), midnight))
        return self._get_events(start, end, timezone, categories=[])

    def get_events(self, start, end, timezone=None, categories=[]):
        """Get all events between `start` and `end` with one of `categories`.

        `start` and `end` have to be datetime objects.
        `categories` is a list of category titles.
        `start` is part of the interval, but `end` is not.
        """
        timezone = self._timezone_name_to_timezone(timezone)
        return self._get_events(start, end, timezone, categories)

    def _get_events(self, start, end, timezone, categories):
        """Get all events between `start` and `end`.

        `start` is part of the interval, but `end` is not.
        `categories` is a list of category titles.
        Only return events of the given `categories`.
        If `categories` is an empty list, do not restrict by category.
        """
        recurring_events = zope.component.getUtility(
            icemac.ab.calendar.interfaces.IRecurringEvents).get_events(
                categories)
        recurred_events = [x.get_events(start, end, timezone)
                           for x in recurring_events]
        events_map = {(x.category, x.in_timezone(timezone)): x
                      for x in itertools.chain(*recurred_events)}
        catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
        query = {
            DATE_INDEX: {'between': (start, end, False, True)},
            'schema_name': {'any_of': [
                ISchemaName(IEvent).schema_name,
                ISchemaName(IRecurringEvent).schema_name,
            ]},
        }
        if categories:
            query['keywords'] = {'any_of': categories}

        # The values for the index are: min, max, min_exclude, max_exclude
        single_events = catalog.searchResults(**query)
        # Sort deleted events first. This way a recurred event can be deleted
        # and later on replaced by a new event of the same category.
        sorted_single_events = sorted(
            single_events, key=lambda x: int(x.deleted), reverse=True)
        # A single_event with the same category and datetime overwrites the
        # recurred event as it is its customization:
        single_events_map = {(x.category, x.in_timezone(timezone)): x
                             for x in sorted_single_events}
        events_map.update(single_events_map)
        # Filter out deleted recurred events and sort:
        return sorted(
            filter(lambda x: not x.deleted, events_map.values()),
            key=lambda x: (x.in_timezone(timezone),
                           icemac.addressbook.interfaces.ITitle(
                               x.category, None)))

    def _timezone_name_to_timezone(self, name):
        """Return a timezone object. If `name` is None, return UTC."""
        if name is None:
            timezone = pytz.utc
        else:
            timezone = pytz.timezone(name)
        return timezone


class CalendarDisplaySettings(grok.Annotation):
    """Store calendar display settings in annotations."""

    grok.context(icemac.ab.calendar.interfaces.ICalendar)
    grok.implements(icemac.ab.calendar.interfaces.ICalendarDisplaySettings)

    person_keyword = gocept.reference.Reference(
        'person_keyword', ensure_integrity=True)
    event_additional_fields = icemac.ab.calendar.property.AddressBookField(
        '_event_additional_fields', multiple=True)

    def __init__(self, *args, **kw):
        super(CalendarDisplaySettings, self).__init__(*args, **kw)
        self.person_keyword = None


@grok.adapter(icemac.addressbook.interfaces.IAddressBook)
@grok.implementer(icemac.ab.calendar.interfaces.ICalendar)
def calendar(address_book):
    """Adapt the event to its calendar."""
    return address_book.calendar
